HEAP32[x >>> 2];

HEAP8[x >>> 0];

HEAP8.length;

HEAP16.set(foo);

HEAP16.set(foo, x >>> 0);

HEAPU32.copyWithin(x >>> 0, y >>> 0);

HEAPU32.copyWithin(x >>> 0, y >>> 0, z >>> 0);

HEAPU8.subarray();

HEAPU8.subarray(x >>> 0);

HEAPU8.subarray(x >>> 0, y >>> 0);

process.versions.node;

insideCall(HEAP32[x >>> 2]);

heap[x >>> 0];

HEAP[x >>> 0];

HeAp[x];
