module.exports={A:{A:{"2436":"J E F G A B PC"},B:{"260":"N O","2436":"C K L H M","8196":"0 1 2 3 4 5 6 P Q R S T U V W X Y Z a b c d e f g h i j k l m n t u v w x y z D"},C:{"2":"7 8 QC 4B I J E F G A B C K L H M N O o p SC TC","772":"9 q r AB BB CB DB EB FB GB HB IB JB KB LB MB NB OB PB","4100":"0 1 2 3 4 5 6 QB RB SB TB UB VB WB XB YB ZB aB bB cB dB eB fB gB hB 5B iB 6B jB kB lB mB nB oB pB qB rB sB tB s uB vB wB xB yB P Q R 7B S T U V W X Y Z a b c d e f g h i j k l m n t u v w x y z D 8B 9B AC RC"},D:{"2":"7 I J E F G A B C","2564":"8 9 K L H M N O o p q r AB BB CB DB EB FB GB HB IB JB KB LB MB NB OB PB QB RB","8196":"0 1 2 3 4 5 6 hB 5B iB 6B jB kB lB mB nB oB pB qB rB sB tB s uB vB wB xB yB P Q R S T U V W X Y Z a b c d e f g h i j k l m n t u v w x y z D 8B 9B AC","10244":"SB TB UB VB WB XB YB ZB aB bB cB dB eB fB gB"},E:{"1":"C K L H 0B DC ZC aC EC FC 1B bC 2B GC HC IC JC KC cC 3B LC MC dC","16":"UC BC","2308":"A B CC zB","2820":"7 I J E F G VC WC XC YC"},F:{"2":"G B eC fC gC hC zB NC iC","16":"C","516":"0B","2564":"8 9 H M N O o p q r AB BB CB DB EB","8196":"UB VB WB XB YB ZB aB bB cB dB eB fB gB hB iB jB kB lB mB nB oB pB qB rB sB tB s uB vB wB xB yB P Q R 7B S T U V W X Y Z a b c d e f g h i j k l m n","10244":"FB GB HB IB JB KB LB MB NB OB PB QB RB SB TB"},G:{"1":"uC vC wC xC yC zC 0C 1C 2C EC FC 1B 3C 2B GC HC IC JC KC 4C 3B LC MC","2":"BC jC OC","2820":"F kC lC mC nC oC pC qC rC sC tC"},H:{"2":"5C"},I:{"2":"4B I 6C 7C 8C 9C OC","260":"D","2308":"AD BD"},J:{"2":"E","2308":"A"},K:{"2":"A B C zB NC","16":"0B","8196":"s"},L:{"8196":"D"},M:{"1028":"D"},N:{"2":"A B"},O:{"8196":"1B"},P:{"2052":"CD DD","2308":"I","8196":"o p q r ED FD GD CC HD ID JD KD LD 2B 3B MD ND"},Q:{"8196":"DC"},R:{"8196":"OD"},S:{"4100":"PD QD"}},B:5,C:"Synchronous Clipboard API",D:true};
