module.exports={A:{A:{"2":"J E F PC","2561":"A B","2692":"G"},B:{"1":"0 1 2 3 4 5 6 P Q R S T U V W X Y Z a b c d e f g h i j k l m n t u v w x y z D","2561":"C K L H M N O"},C:{"1":"0 1 2 3 4 5 6 YB ZB aB bB cB dB eB fB gB hB 5B iB 6B jB kB lB mB nB oB pB qB rB sB tB s uB vB wB xB yB P Q R 7B S T U V W X Y Z a b c d e f g h i j k l m n t u v w x y z D 8B 9B AC RC","16":"QC","1537":"7 8 9 I J E F G A B C K L H M N O o p q r AB BB CB DB EB FB GB HB IB JB KB LB MB NB OB PB QB RB SB TB UB VB WB XB TC","1796":"4B SC"},D:{"1":"0 1 2 3 4 5 6 nB oB pB qB rB sB tB s uB vB wB xB yB P Q R S T U V W X Y Z a b c d e f g h i j k l m n t u v w x y z D 8B 9B AC","16":"7 I J E F G A B C K L","1025":"KB LB MB NB OB PB QB RB SB TB UB VB WB XB YB ZB aB bB cB dB eB fB gB hB 5B iB 6B jB kB lB mB","1537":"8 9 H M N O o p q r AB BB CB DB EB FB GB HB IB JB"},E:{"1":"L H DC ZC aC EC FC 1B bC 2B GC HC IC JC KC cC 3B LC MC dC","16":"7 I J UC BC","1025":"E F G A B C WC XC YC CC zB","1537":"VC","4097":"K 0B"},F:{"1":"bB cB dB eB fB gB hB iB jB kB lB mB nB oB pB qB rB sB tB s uB vB wB xB yB P Q R 7B S T U V W X Y Z a b c d e f g h i j k l m n 0B","16":"G B C eC fC gC hC zB NC","260":"iC","1025":"9 q r AB BB CB DB EB FB GB HB IB JB KB LB MB NB OB PB QB RB SB TB UB VB WB XB YB ZB aB","1537":"8 H M N O o p"},G:{"1":"xC yC zC 0C 1C 2C EC FC 1B 3C 2B GC HC IC JC KC 4C 3B LC MC","16":"BC jC OC","1025":"F nC oC pC qC rC sC tC uC","1537":"kC lC mC","4097":"vC wC"},H:{"2":"5C"},I:{"16":"6C 7C","1025":"D BD","1537":"4B I 8C 9C OC AD"},J:{"1025":"A","1537":"E"},K:{"1":"A B C s zB NC 0B"},L:{"1":"D"},M:{"1":"D"},N:{"2561":"A B"},O:{"1":"1B"},P:{"1025":"I o p q r CD DD ED FD GD CC HD ID JD KD LD 2B 3B MD ND"},Q:{"1":"DC"},R:{"1":"OD"},S:{"1":"QD","1537":"PD"}},B:1,C:"input event",D:true};
