module.exports={A:{A:{"2":"J E F G A B PC"},B:{"1":"0 1 2 3 4 5 6 W X Y Z a b c d e f g h i j k l m n t u v w x y z D","2":"C K L H M N O","322":"P Q R S T","578":"U V"},C:{"1":"0 1 2 3 4 5 6 X Y Z a b c d e f g h i j k l m n t u v w x y z D 8B 9B AC RC","2":"7 8 9 QC 4B I J E F G A B C K L H M N O o p q r AB BB CB DB EB FB GB HB IB JB KB LB MB NB OB PB QB RB SB TB UB VB WB XB YB ZB aB bB cB dB eB fB gB hB 5B iB 6B jB kB lB mB nB oB pB qB rB sB SC TC","194":"tB s uB vB wB xB yB P Q R 7B S T U V W"},D:{"1":"0 1 2 3 4 5 6 W X Y Z a b c d e f g h i j k l m n t u v w x y z D 8B 9B AC","2":"7 8 9 I J E F G A B C K L H M N O o p q r AB BB CB DB EB FB GB HB IB JB KB LB MB NB OB PB QB RB SB TB UB VB WB XB YB ZB aB bB cB dB eB fB gB hB 5B iB 6B jB kB lB mB nB oB pB qB rB sB tB s uB vB wB xB yB","322":"P Q R S T","578":"U V"},E:{"2":"7 I J E F G A B C K UC BC VC WC XC YC CC zB 0B DC","2052":"JC KC cC 3B LC MC dC","2116":"2B GC HC IC","3140":"L H ZC aC EC FC 1B bC"},F:{"1":"uB vB wB xB yB P Q R 7B S T U V W X Y Z a b c d e f g h i j k l m n","2":"8 9 G B C H M N O o p q r AB BB CB DB EB FB GB HB IB JB KB LB MB NB OB PB QB RB SB TB UB VB WB XB YB ZB aB bB cB dB eB fB gB hB iB jB kB lB mB nB oB pB qB rB sB tB eC fC gC hC zB NC iC 0B","578":"s"},G:{"2":"F BC jC OC kC lC mC nC oC pC qC rC sC tC uC vC wC xC yC zC","2052":"JC KC 4C 3B LC MC","2116":"0C 1C 2C EC FC 1B 3C 2B GC HC IC"},H:{"2":"5C"},I:{"1":"D","2":"4B I 6C 7C 8C 9C OC AD BD"},J:{"2":"E A"},K:{"1":"s","2":"A B C zB NC 0B"},L:{"1":"D"},M:{"1":"D"},N:{"2":"A B"},O:{"1":"1B"},P:{"1":"o p q r KD LD 2B 3B MD ND","2":"I CD DD ED FD GD CC HD ID JD"},Q:{"2":"DC"},R:{"1":"OD"},S:{"2":"PD QD"}},B:6,C:"HTTP/3 protocol",D:true};
