module.exports={A:{A:{"2":"J E F G A B PC"},B:{"2":"C K L H M N O","1025":"0 1 2 3 4 5 6 d e f g h i j k l m n t u v w x y z D","1537":"P Q R S T U V W X Y Z a b c"},C:{"2":"QC","932":"7 8 9 4B I J E F G A B C K L H M N O o p q r AB BB CB DB EB FB GB HB IB JB KB LB MB NB OB PB QB RB SB TB UB VB WB XB YB ZB aB bB cB dB eB fB gB hB 5B iB 6B jB kB lB mB SC TC","2308":"0 1 2 3 4 5 6 nB oB pB qB rB sB tB s uB vB wB xB yB P Q R 7B S T U V W X Y Z a b c d e f g h i j k l m n t u v w x y z D 8B 9B AC RC"},D:{"2":"7 8 I J E F G A B C K L H M N O o p","545":"9 q r AB BB CB DB EB FB GB HB IB JB KB LB MB NB OB PB QB RB SB TB UB","1025":"0 1 2 3 4 5 6 d e f g h i j k l m n t u v w x y z D 8B 9B AC","1537":"VB WB XB YB ZB aB bB cB dB eB fB gB hB 5B iB 6B jB kB lB mB nB oB pB qB rB sB tB s uB vB wB xB yB P Q R S T U V W X Y Z a b c"},E:{"1":"2B GC HC IC JC KC cC 3B LC MC dC","2":"7 I J UC BC VC","516":"B C K L H zB 0B DC ZC aC EC FC 1B bC","548":"G A YC CC","676":"E F WC XC"},F:{"2":"G B C eC fC gC hC zB NC iC 0B","513":"JB","545":"8 9 H M N O o p q r AB BB CB DB EB FB GB HB","1025":"e f g h i j k l m n","1537":"IB KB LB MB NB OB PB QB RB SB TB UB VB WB XB YB ZB aB bB cB dB eB fB gB hB iB jB kB lB mB nB oB pB qB rB sB tB s uB vB wB xB yB P Q R 7B S T U V W X Y Z a b c d"},G:{"1":"2B GC HC IC JC KC 4C 3B LC MC","2":"BC jC OC kC lC","516":"0C 1C 2C EC FC 1B 3C","548":"oC pC qC rC sC tC uC vC wC xC yC zC","676":"F mC nC"},H:{"2":"5C"},I:{"2":"4B I 6C 7C 8C 9C OC","545":"AD BD","1025":"D"},J:{"2":"E","545":"A"},K:{"2":"A B C zB NC 0B","1025":"s"},L:{"1025":"D"},M:{"2308":"D"},N:{"2":"A B"},O:{"1537":"1B"},P:{"545":"I","1025":"o p q r 3B MD ND","1537":"CD DD ED FD GD CC HD ID JD KD LD 2B"},Q:{"1537":"DC"},R:{"1537":"OD"},S:{"932":"PD","2308":"QD"}},B:5,C:"Intrinsic & Extrinsic Sizing",D:true};
