module.exports={A:{A:{"2":"J E F G A B PC"},B:{"1":"H M N O","33":"C K L","164":"0 1 2 3 4 5 6 P Q R S T U V W X Y Z a b c d e f g h i j k l m n t u v w x y z D"},C:{"1":"0 1 2 3 4 5 6 YB ZB aB bB cB dB eB fB gB hB 5B iB 6B jB kB lB mB nB oB pB qB rB sB tB s uB vB wB xB yB P Q R 7B S T U V W X Y Z a b c d e f g h i j k l m n t u v w x y z D 8B 9B AC RC","2":"7 8 9 QC 4B I J E F G A B C K L H M N O o p q r AB BB CB DB EB FB GB HB IB JB KB LB MB NB OB PB QB RB SB TB UB VB WB XB SC TC"},D:{"132":"8B 9B AC","164":"0 1 2 3 4 5 6 7 8 9 I J E F G A B C K L H M N O o p q r AB BB CB DB EB FB GB HB IB JB KB LB MB NB OB PB QB RB SB TB UB VB WB XB YB ZB aB bB cB dB eB fB gB hB 5B iB 6B jB kB lB mB nB oB pB qB rB sB tB s uB vB wB xB yB P Q R S T U V W X Y Z a b c d e f g h i j k l m n t u v w x y z D"},E:{"16":"UC BC","132":"1B bC 2B GC HC IC JC KC cC 3B LC MC dC","388":"L H ZC aC EC FC","420":"7 I J E F G A B C K VC WC XC YC CC zB 0B DC"},F:{"2":"G B C eC fC gC hC zB NC iC 0B","164":"8 9 H M N O o p q r AB BB CB DB EB FB GB HB IB JB KB LB MB NB OB PB QB RB SB TB UB VB WB XB YB ZB aB bB cB dB eB fB gB hB iB jB kB lB mB nB oB pB qB rB sB tB s uB vB wB xB yB P Q R 7B S T U V W X Y Z a b c d e f g h i j k l m n"},G:{"16":"BC jC OC kC","132":"1B 3C 2B GC HC IC JC KC 4C 3B LC MC","388":"0C 1C 2C EC FC","420":"F lC mC nC oC pC qC rC sC tC uC vC wC xC yC zC"},H:{"2":"5C"},I:{"16":"4B 6C 7C 8C","164":"I D 9C OC AD BD"},J:{"164":"E A"},K:{"16":"A B C zB NC 0B","164":"s"},L:{"164":"D"},M:{"1":"D"},N:{"2":"A B"},O:{"164":"1B"},P:{"164":"I o p q r CD DD ED FD GD CC HD ID JD KD LD 2B 3B MD ND"},Q:{"164":"DC"},R:{"164":"OD"},S:{"1":"PD QD"}},B:7,C:"Background-clip: text",D:true};
