module.exports={A:{A:{"644":"J E F G PC","772":"A B"},B:{"1":"0 1 2 3 4 5 6 O P Q R S T U V W X Y Z a b c d e f g h i j k l m n t u v w x y z D","260":"C K L H M N"},C:{"1":"0 1 2 3 4 5 6 7 8 9 I J E F G A B C K L H M N O o p q r AB BB CB DB EB FB GB HB IB JB KB LB MB NB OB PB QB RB SB TB UB VB WB XB YB ZB aB bB cB dB eB fB gB hB 5B iB 6B jB kB lB mB nB oB pB qB rB sB tB s uB vB wB xB yB P Q R 7B S T U V W X Y Z a b c d e f g h i j k l m n t u v w x y z D 8B 9B AC RC SC TC","8":"QC 4B"},D:{"1":"0 1 2 3 4 5 6 7 8 9 I J E F G A B C K L H M N O o p q r AB BB CB DB EB FB GB HB IB JB KB LB MB NB OB PB QB RB SB TB UB VB WB XB YB ZB aB bB cB dB eB fB gB hB 5B iB 6B jB kB lB mB nB oB pB qB rB sB tB s uB vB wB xB yB P Q R S T U V W X Y Z a b c d e f g h i j k l m n t u v w x y z D 8B 9B AC"},E:{"1":"7 I J E F G A B C K L H UC BC VC WC XC YC CC zB 0B DC ZC aC EC FC 1B bC 2B GC HC IC JC KC cC 3B LC MC dC"},F:{"1":"8 9 C H M N O o p q r AB BB CB DB EB FB GB HB IB JB KB LB MB NB OB PB QB RB SB TB UB VB WB XB YB ZB aB bB cB dB eB fB gB hB iB jB kB lB mB nB oB pB qB rB sB tB s uB vB wB xB yB P Q R 7B S T U V W X Y Z a b c d e f g h i j k l m n 0B","8":"G B eC fC gC hC zB NC iC"},G:{"1":"2C EC FC 1B 3C 2B GC HC IC JC KC 4C 3B LC MC","2":"F BC jC OC kC lC mC nC oC pC qC rC sC tC uC vC wC xC yC zC 0C 1C"},H:{"2":"5C"},I:{"2":"4B I 6C 7C 8C 9C OC AD BD","1025":"D"},J:{"2":"E A"},K:{"1":"0B","8":"A B C zB NC","1025":"s"},L:{"1025":"D"},M:{"2":"D"},N:{"1":"A B"},O:{"1025":"1B"},P:{"2":"I o p q r CD DD ED FD GD CC HD ID JD KD LD 2B 3B MD ND"},Q:{"1":"DC"},R:{"2":"OD"},S:{"2":"PD QD"}},B:1,C:"Drag and Drop",D:true};
