module.exports={A:{A:{"2":"J E F G PC","8":"A B"},B:{"1":"P","2":"0 1 2 3 4 5 6 Q R S T U V W X Y Z a b c d e f g h i j k l m n t u v w x y z D","8":"C K L H M N O"},C:{"2":"0 1 2 3 4 5 6 7 8 QC 4B I J E F G A B C K L H M N O o p q 5B iB 6B jB kB lB mB nB oB pB qB rB sB tB s uB vB wB xB yB P Q R 7B S T U V W X Y Z a b c d e f g h i j k l m n t u v w x y z D 8B 9B AC RC SC TC","66":"9 r AB BB CB DB EB","72":"FB GB HB IB JB KB LB MB NB OB PB QB RB SB TB UB VB WB XB YB ZB aB bB cB dB eB fB gB hB"},D:{"1":"IB JB KB LB MB NB OB PB QB RB SB TB UB VB WB XB YB ZB aB bB cB dB eB fB gB hB 5B iB 6B jB kB lB mB nB oB pB qB rB sB tB s uB vB wB xB yB P","2":"0 1 2 3 4 5 6 7 8 9 I J E F G A B C K L H M N O o p q r AB BB Q R S T U V W X Y Z a b c d e f g h i j k l m n t u v w x y z D 8B 9B AC","66":"CB DB EB FB GB HB"},E:{"2":"7 I UC BC VC","8":"J E F G A B C K L H WC XC YC CC zB 0B DC ZC aC EC FC 1B bC 2B GC HC IC JC KC cC 3B LC MC dC"},F:{"1":"9 o p q r AB BB CB DB EB FB GB HB IB JB KB LB MB NB OB PB QB RB SB TB UB VB WB XB YB ZB aB bB cB dB eB fB gB hB iB jB kB lB mB nB","2":"G B C oB pB qB rB sB tB s uB vB wB xB yB P Q R 7B S T U V W X Y Z a b c d e f g h i j k l m n eC fC gC hC zB NC iC 0B","66":"8 H M N O"},G:{"2":"BC jC OC kC lC","8":"F mC nC oC pC qC rC sC tC uC vC wC xC yC zC 0C 1C 2C EC FC 1B 3C 2B GC HC IC JC KC 4C 3B LC MC"},H:{"2":"5C"},I:{"1":"BD","2":"4B I D 6C 7C 8C 9C OC AD"},J:{"2":"E A"},K:{"2":"A B C s zB NC 0B"},L:{"2":"D"},M:{"2":"D"},N:{"2":"A B"},O:{"1":"1B"},P:{"1":"I CD DD ED FD GD CC HD ID","2":"o p q r JD KD LD 2B 3B MD ND"},Q:{"1":"DC"},R:{"2":"OD"},S:{"2":"QD","72":"PD"}},B:7,C:"Custom Elements (deprecated V0 spec)",D:true};
