/*
 * WebGPU globals
 */
var GPU;
var GPUAdapter;
var GPUAdapterInfo;
var GPUBindGroup;
var GPUBindGroupLayout;
var GPUBuffer;
var GPUBufferUsage;
var GPUCanvasContext;
var GPUColorWrite;
var GPUCommandBuffer;
var GPUCommandEncoder;
var GPUCompilationInfo;
var GPUCompilationMessage;
var GPUComputePassEncoder;
var GPUComputePipeline;
var GPUDevice;
var GPUDeviceLostInfo;
var GPUError;
var GPUExternalTexture;
var GPUInternalError;
var GPUMapMode;
var GPUOutOfMemoryError;
var GPUPipelineLayout;
var GPUQuerySet;
var GPUQueue;
var GPURenderBundle;
var GPURenderBundleEncoder;
var GPURenderPassEncoder;
var GPURenderPipeline;
var GPUSampler;
var GPUShaderModule;
var GPUShaderStage;
var GPUSupportedFeatures;
var GPUSupportedLimits;
var GPUTexture;
var GPUTextureUsage;
var GPUTextureView;
var GPUUncapturedErrorEvent;
var GPUValidationError;
