module.exports={A:{A:{"2":"J E F PC","520":"G A B"},B:{"1":"0 1 2 3 4 5 6 P Q R S T U V W X Y Z a b c d e f g h i j k l m n t u v w x y z D","8":"C K","388":"L H M N O"},C:{"1":"0 1 2 3 4 5 6 DB EB FB GB HB IB JB KB LB MB NB OB PB QB RB SB TB UB VB WB XB YB ZB aB bB cB dB eB fB gB hB 5B iB 6B jB kB lB mB nB oB pB qB rB sB tB s uB vB wB xB yB P Q R 7B S T U V W X Y Z a b c d e f g h i j k l m n t u v w x y z D 8B 9B AC RC","2":"QC 4B SC TC","132":"7 8 9 I J E F G A B C K L H M N O o p q r AB BB CB"},D:{"1":"0 1 2 3 4 5 6 AB BB CB DB EB FB GB HB IB JB KB LB MB NB OB PB QB RB SB TB UB VB WB XB YB ZB aB bB cB dB eB fB gB hB 5B iB 6B jB kB lB mB nB oB pB qB rB sB tB s uB vB wB xB yB P Q R S T U V W X Y Z a b c d e f g h i j k l m n t u v w x y z D 8B 9B AC","2":"7 I","132":"8 9 J E F G A B C K L H M N O o p q r"},E:{"1":"2B GC HC IC JC KC cC 3B LC MC dC","2":"UC","8":"7 I BC VC","520":"J E F G A B C WC XC YC CC zB","1028":"K 0B DC","7172":"L","8196":"H ZC aC EC FC 1B bC"},F:{"1":"8 9 M N O o p q r AB BB CB DB EB FB GB HB IB JB KB LB MB NB OB PB QB RB SB TB UB VB WB XB YB ZB aB bB cB dB eB fB gB hB iB jB kB lB mB nB oB pB qB rB sB tB s uB vB wB xB yB P Q R 7B S T U V W X Y Z a b c d e f g h i j k l m n","2":"G eC fC gC","132":"B C H hC zB NC iC 0B"},G:{"2":"F BC jC OC kC lC mC nC oC pC qC rC sC tC uC","1028":"vC wC xC yC zC","3076":"0C 1C 2C EC FC 1B 3C 2B GC HC IC JC KC 4C 3B LC MC"},H:{"2":"5C"},I:{"1":"D","2":"6C 7C","132":"4B I 8C 9C OC AD BD"},J:{"2":"E A"},K:{"1":"s","2":"A B C zB NC 0B"},L:{"1":"D"},M:{"1":"D"},N:{"8":"A B"},O:{"1":"1B"},P:{"1":"o p q r CD DD ED FD GD CC HD ID JD KD LD 2B 3B MD ND","132":"I"},Q:{"1":"DC"},R:{"1":"OD"},S:{"1":"PD QD"}},B:6,C:"WebM video format",D:true};
