/* Auto-generated by tools/gen_sig_info.py. DO NOT EDIT. */

sigs = {
  IMG_Init__sig: 'ii',
  IMG_Load__sig: 'pp',
  IMG_Load_RW__sig: 'ppi',
  IMG_Quit__sig: 'v',
  Mix_AllocateChannels__sig: 'ii',
  Mix_ChannelFinished__sig: 'vp',
  Mix_CloseAudio__sig: 'v',
  Mix_FadeInChannelTimed__sig: 'iipiii',
  Mix_FadeInMusicPos__sig: 'ipiid',
  Mix_FadeOutChannel__sig: 'iii',
  Mix_FadeOutMusic__sig: 'ii',
  Mix_FadingChannel__sig: 'ii',
  Mix_FreeChunk__sig: 'vp',
  Mix_FreeMusic__sig: 'vp',
  Mix_HaltChannel__sig: 'ii',
  Mix_HaltMusic__sig: 'i',
  Mix_HookMusicFinished__sig: 'vp',
  Mix_Init__sig: 'ii',
  Mix_Linked_Version__sig: 'p',
  Mix_LoadMUS__sig: 'pp',
  Mix_LoadMUS_RW__sig: 'pp',
  Mix_LoadWAV__sig: 'pp',
  Mix_LoadWAV_RW__sig: 'ppi',
  Mix_OpenAudio__sig: 'iiiii',
  Mix_Pause__sig: 'vi',
  Mix_PauseMusic__sig: 'v',
  Mix_Paused__sig: 'ii',
  Mix_PausedMusic__sig: 'i',
  Mix_PlayChannelTimed__sig: 'iipii',
  Mix_PlayMusic__sig: 'ipi',
  Mix_Playing__sig: 'ii',
  Mix_PlayingMusic__sig: 'i',
  Mix_QuerySpec__sig: 'ippp',
  Mix_QuickLoad_RAW__sig: 'ppi',
  Mix_Quit__sig: 'v',
  Mix_ReserveChannels__sig: 'ii',
  Mix_Resume__sig: 'vi',
  Mix_ResumeMusic__sig: 'v',
  Mix_SetPanning__sig: 'iiii',
  Mix_SetPosition__sig: 'iiii',
  Mix_SetPostMix__sig: 'vpp',
  Mix_Volume__sig: 'iii',
  Mix_VolumeChunk__sig: 'ipi',
  Mix_VolumeMusic__sig: 'ii',
  SDL_AddTimer__sig: 'iipp',
  SDL_AllocRW__sig: 'p',
  SDL_AudioDriverName__sig: 'ppi',
  SDL_AudioQuit__sig: 'v',
  SDL_ClearError__sig: 'v',
  SDL_CloseAudio__sig: 'v',
  SDL_CondBroadcast__sig: 'ip',
  SDL_CondSignal__sig: 'ip',
  SDL_CondWait__sig: 'ipp',
  SDL_CondWaitTimeout__sig: 'ippi',
  SDL_ConvertSurface__sig: 'pppi',
  SDL_CreateCond__sig: 'p',
  SDL_CreateMutex__sig: 'p',
  SDL_CreateRGBSurface__sig: 'piiiiiiii',
  SDL_CreateRGBSurfaceFrom__sig: 'ppiiiiiiii',
  SDL_CreateThread__sig: 'ppp',
  SDL_Delay__sig: 'vi',
  SDL_DestroyCond__sig: 'vp',
  SDL_DestroyMutex__sig: 'vp',
  SDL_DestroyRenderer__sig: 'vp',
  SDL_DestroyWindow__sig: 'vp',
  SDL_DisplayFormatAlpha__sig: 'pp',
  SDL_EnableKeyRepeat__sig: 'iii',
  SDL_EnableUNICODE__sig: 'ii',
  SDL_FillRect__sig: 'ippi',
  SDL_Flip__sig: 'ip',
  SDL_FreeRW__sig: 'vp',
  SDL_FreeSurface__sig: 'vp',
  SDL_GL_DeleteContext__sig: 'vp',
  SDL_GL_ExtensionSupported__sig: 'ip',
  SDL_GL_GetAttribute__sig: 'iip',
  SDL_GL_GetProcAddress__sig: 'pp',
  SDL_GL_GetSwapInterval__sig: 'i',
  SDL_GL_MakeCurrent__sig: 'ipp',
  SDL_GL_SetAttribute__sig: 'iii',
  SDL_GL_SetSwapInterval__sig: 'ii',
  SDL_GL_SwapBuffers__sig: 'v',
  SDL_GL_SwapWindow__sig: 'vp',
  SDL_GetAppState__sig: 'i',
  SDL_GetAudioDriver__sig: 'pi',
  SDL_GetClipRect__sig: 'vpp',
  SDL_GetCurrentAudioDriver__sig: 'p',
  SDL_GetError__sig: 'p',
  SDL_GetKeyName__sig: 'pi',
  SDL_GetKeyboardState__sig: 'pp',
  SDL_GetModState__sig: 'i',
  SDL_GetMouseState__sig: 'ipp',
  SDL_GetNumAudioDrivers__sig: 'i',
  SDL_GetRGB__sig: 'vipppp',
  SDL_GetRGBA__sig: 'vippppp',
  SDL_GetScancodeFromKey__sig: 'ii',
  SDL_GetThreadID__sig: 'pp',
  SDL_GetTicks__sig: 'i',
  SDL_GetVideoInfo__sig: 'p',
  SDL_GetVideoSurface__sig: 'p',
  SDL_GetWindowFlags__sig: 'ip',
  SDL_GetWindowSize__sig: 'vppp',
  SDL_Has3DNow__sig: 'i',
  SDL_Has3DNowExt__sig: 'i',
  SDL_HasAltiVec__sig: 'i',
  SDL_HasMMX__sig: 'i',
  SDL_HasMMXExt__sig: 'i',
  SDL_HasRDTSC__sig: 'i',
  SDL_HasSSE__sig: 'i',
  SDL_HasSSE2__sig: 'i',
  SDL_Init__sig: 'ii',
  SDL_InitSubSystem__sig: 'ii',
  SDL_JoystickClose__sig: 'vp',
  SDL_JoystickEventState__sig: 'ii',
  SDL_JoystickGetAxis__sig: 'ipi',
  SDL_JoystickGetBall__sig: 'ipipp',
  SDL_JoystickGetButton__sig: 'ipi',
  SDL_JoystickGetHat__sig: 'ipi',
  SDL_JoystickIndex__sig: 'ip',
  SDL_JoystickName__sig: 'pi',
  SDL_JoystickNumAxes__sig: 'ip',
  SDL_JoystickNumBalls__sig: 'ip',
  SDL_JoystickNumButtons__sig: 'ip',
  SDL_JoystickNumHats__sig: 'ip',
  SDL_JoystickOpen__sig: 'pi',
  SDL_JoystickOpened__sig: 'ii',
  SDL_JoystickUpdate__sig: 'v',
  SDL_Linked_Version__sig: 'p',
  SDL_ListModes__sig: 'ppi',
  SDL_LoadBMP_RW__sig: 'ppi',
  SDL_LockAudio__sig: 'v',
  SDL_LockSurface__sig: 'ip',
  SDL_LogSetOutputFunction__sig: 'vpp',
  SDL_LowerBlit__sig: 'ipppp',
  SDL_LowerBlitScaled__sig: 'ipppp',
  SDL_MapRGB__sig: 'ipiii',
  SDL_MapRGBA__sig: 'ipiiii',
  SDL_NumJoysticks__sig: 'i',
  SDL_OpenAudio__sig: 'ipp',
  SDL_PauseAudio__sig: 'vi',
  SDL_PeepEvents__sig: 'ipiiii',
  SDL_PollEvent__sig: 'ip',
  SDL_PumpEvents__sig: 'v',
  SDL_PushEvent__sig: 'ip',
  SDL_Quit__sig: 'v',
  SDL_QuitSubSystem__sig: 'vi',
  SDL_RWFromConstMem__sig: 'ppi',
  SDL_RWFromFile__sig: 'ppp',
  SDL_RWFromMem__sig: 'ppi',
  SDL_RemoveTimer__sig: 'ii',
  SDL_SaveBMP_RW__sig: 'ippi',
  SDL_SetAlpha__sig: 'ipii',
  SDL_SetClipRect__sig: 'ipp',
  SDL_SetColorKey__sig: 'ipii',
  SDL_SetColors__sig: 'ippii',
  SDL_SetError__sig: 'vpp',
  SDL_SetGamma__sig: 'ifff',
  SDL_SetGammaRamp__sig: 'ippp',
  SDL_SetPalette__sig: 'ipipii',
  SDL_SetVideoMode__sig: 'piiii',
  SDL_SetWindowFullscreen__sig: 'ipi',
  SDL_SetWindowTitle__sig: 'vpp',
  SDL_ShowCursor__sig: 'ii',
  SDL_StartTextInput__sig: 'v',
  SDL_StopTextInput__sig: 'v',
  SDL_ThreadID__sig: 'p',
  SDL_UnlockAudio__sig: 'v',
  SDL_UnlockSurface__sig: 'vp',
  SDL_UpdateRect__sig: 'vpiiii',
  SDL_UpdateRects__sig: 'vpip',
  SDL_UpperBlit__sig: 'ipppp',
  SDL_UpperBlitScaled__sig: 'ipppp',
  SDL_VideoDriverName__sig: 'ppi',
  SDL_VideoModeOK__sig: 'iiiii',
  SDL_VideoQuit__sig: 'v',
  SDL_WM_GrabInput__sig: 'ii',
  SDL_WM_IconifyWindow__sig: 'i',
  SDL_WM_SetCaption__sig: 'vpp',
  SDL_WM_SetIcon__sig: 'vpp',
  SDL_WM_ToggleFullScreen__sig: 'ip',
  SDL_WaitThread__sig: 'vpp',
  SDL_WarpMouse__sig: 'vii',
  SDL_WasInit__sig: 'ii',
  SDL_mutexP__sig: 'ip',
  SDL_mutexV__sig: 'ip',
  TTF_CloseFont__sig: 'vp',
  TTF_FontAscent__sig: 'ip',
  TTF_FontDescent__sig: 'ip',
  TTF_FontHeight__sig: 'ip',
  TTF_FontLineSkip__sig: 'ip',
  TTF_GlyphMetrics__sig: 'ipippppp',
  TTF_Init__sig: 'i',
  TTF_OpenFont__sig: 'ppi',
  TTF_Quit__sig: 'v',
  TTF_RenderText_Blended__sig: 'pppp',
  TTF_RenderText_Shaded__sig: 'ppppp',
  TTF_RenderText_Solid__sig: 'pppp',
  TTF_RenderUTF8_Solid__sig: 'pppp',
  TTF_SizeText__sig: 'ipppp',
  TTF_SizeUTF8__sig: 'ipppp',
  XChangeWindowAttributes__sig: 'ipppp',
  XCreateWindow__sig: 'pppiiiiiiippp',
  XInternAtom__sig: 'pppi',
  XMapWindow__sig: 'ipp',
  XOpenDisplay__sig: 'pp',
  XPending__sig: 'ip',
  XSendEvent__sig: 'ippipp',
  XSetWMHints__sig: 'ippp',
  XStoreName__sig: 'ippp',
  _Unwind_Backtrace__sig: 'ipp',
  _Unwind_DeleteException__sig: 'vp',
  _Unwind_FindEnclosingFunction__sig: 'pp',
  _Unwind_GetIPInfo__sig: 'ppp',
  _Unwind_RaiseException__sig: 'ip',
  __asctime_r__sig: 'ppp',
  __assert_fail__sig: 'vppip',
  __call_sighandler__sig: 'vpi',
  __cxa_begin_catch__sig: 'pp',
  __cxa_call_unexpected__sig: 'vp',
  __cxa_current_primary_exception__sig: 'p',
  __cxa_end_catch__sig: 'v',
  __cxa_get_exception_ptr__sig: 'pp',
  __cxa_rethrow__sig: 'v',
  __cxa_rethrow_primary_exception__sig: 'vp',
  __cxa_throw__sig: 'vppp',
  __cxa_uncaught_exceptions__sig: 'i',
  __emscripten_init_main_thread_js__sig: 'vp',
  __emscripten_thread_cleanup__sig: 'vp',
  __handle_stack_overflow__sig: 'vp',
  __pthread_create_js__sig: 'ipppp',
  __pthread_kill_js__sig: 'ipi',
  __resumeException__sig: 'vp',
  __syscall__newselect__sig: 'iipppp',
  __syscall_accept4__sig: 'iippiii',
  __syscall_bind__sig: 'iippiii',
  __syscall_chdir__sig: 'ip',
  __syscall_chmod__sig: 'ipi',
  __syscall_connect__sig: 'iippiii',
  __syscall_dup__sig: 'ii',
  __syscall_dup3__sig: 'iiii',
  __syscall_faccessat__sig: 'iipii',
  __syscall_fadvise64__sig: 'iijji',
  __syscall_fallocate__sig: 'iiijj',
  __syscall_fchdir__sig: 'ii',
  __syscall_fchmod__sig: 'iii',
  __syscall_fchmodat__sig: 'iipip',
  __syscall_fchown32__sig: 'iiii',
  __syscall_fchownat__sig: 'iipiii',
  __syscall_fcntl64__sig: 'iiip',
  __syscall_fdatasync__sig: 'ii',
  __syscall_fstat64__sig: 'iip',
  __syscall_fstatfs64__sig: 'iipp',
  __syscall_ftruncate64__sig: 'iij',
  __syscall_getcwd__sig: 'ipp',
  __syscall_getdents64__sig: 'iipp',
  __syscall_getpeername__sig: 'iippiii',
  __syscall_getsockname__sig: 'iippiii',
  __syscall_getsockopt__sig: 'iiiippi',
  __syscall_ioctl__sig: 'iiip',
  __syscall_listen__sig: 'iiiiiii',
  __syscall_lstat64__sig: 'ipp',
  __syscall_mkdirat__sig: 'iipi',
  __syscall_mknodat__sig: 'iipii',
  __syscall_newfstatat__sig: 'iippi',
  __syscall_openat__sig: 'iipip',
  __syscall_pipe__sig: 'ip',
  __syscall_poll__sig: 'ipii',
  __syscall_readlinkat__sig: 'iippp',
  __syscall_recvfrom__sig: 'iippipp',
  __syscall_recvmsg__sig: 'iipiiii',
  __syscall_renameat__sig: 'iipip',
  __syscall_rmdir__sig: 'ip',
  __syscall_sendmsg__sig: 'iipippi',
  __syscall_sendto__sig: 'iippipp',
  __syscall_shutdown__sig: 'iiiiiii',
  __syscall_socket__sig: 'iiiiiii',
  __syscall_stat64__sig: 'ipp',
  __syscall_statfs64__sig: 'ippp',
  __syscall_symlink__sig: 'ipp',
  __syscall_symlinkat__sig: 'ipip',
  __syscall_truncate64__sig: 'ipj',
  __syscall_unlinkat__sig: 'iipi',
  __syscall_utimensat__sig: 'iippi',
  _dlopen_js__sig: 'pp',
  _dlsym_catchup_js__sig: 'ppi',
  _dlsym_js__sig: 'pppp',
  _embind_create_inheriting_constructor__sig: 'pppp',
  _embind_finalize_value_array__sig: 'vp',
  _embind_finalize_value_object__sig: 'vp',
  _embind_register_bigint__sig: 'vpppjj',
  _embind_register_bool__sig: 'vppii',
  _embind_register_class__sig: 'vppppppppppppp',
  _embind_register_class_class_function__sig: 'vppippppi',
  _embind_register_class_class_property__sig: 'vpppppppp',
  _embind_register_class_constructor__sig: 'vpipppp',
  _embind_register_class_function__sig: 'vppippppii',
  _embind_register_class_property__sig: 'vpppppppppp',
  _embind_register_constant__sig: 'vppd',
  _embind_register_emval__sig: 'vpp',
  _embind_register_enum__sig: 'vpppi',
  _embind_register_enum_value__sig: 'vppi',
  _embind_register_float__sig: 'vppp',
  _embind_register_function__sig: 'vpippppi',
  _embind_register_integer__sig: 'vpppii',
  _embind_register_memory_view__sig: 'vpip',
  _embind_register_smart_ptr__sig: 'vpppipppppppp',
  _embind_register_std_string__sig: 'vpp',
  _embind_register_std_wstring__sig: 'vppp',
  _embind_register_user_type__sig: 'vpp',
  _embind_register_value_array__sig: 'vpppppp',
  _embind_register_value_array_element__sig: 'vppppppppp',
  _embind_register_value_object__sig: 'vpppppp',
  _embind_register_value_object_field__sig: 'vpppppppppp',
  _embind_register_void__sig: 'vpp',
  _emscripten_create_wasm_worker__sig: 'ipi',
  _emscripten_dlopen_js__sig: 'vpppp',
  _emscripten_dlsync_threads__sig: 'v',
  _emscripten_dlsync_threads_async__sig: 'vppp',
  _emscripten_fetch_free__sig: 'vi',
  _emscripten_fetch_get_response_headers__sig: 'pipp',
  _emscripten_fetch_get_response_headers_length__sig: 'pi',
  _emscripten_fs_load_embedded_files__sig: 'vp',
  _emscripten_get_now_is_monotonic__sig: 'i',
  _emscripten_get_progname__sig: 'vpi',
  _emscripten_notify_mailbox_postmessage__sig: 'vppp',
  _emscripten_push_main_loop_blocker__sig: 'vppp',
  _emscripten_push_uncounted_main_loop_blocker__sig: 'vppp',
  _emscripten_receive_on_main_thread_js__sig: 'dipip',
  _emscripten_runtime_keepalive_clear__sig: 'v',
  _emscripten_set_offscreencanvas_size__sig: 'ipii',
  _emscripten_thread_exit_joinable__sig: 'vp',
  _emscripten_thread_mailbox_await__sig: 'vp',
  _emscripten_thread_set_strongref__sig: 'vp',
  _emscripten_throw_longjmp__sig: 'v',
  _emval_as__sig: 'dppp',
  _emval_as_int64__sig: 'jpp',
  _emval_as_uint64__sig: 'jpp',
  _emval_await__sig: 'pp',
  _emval_call__sig: 'dpppp',
  _emval_call_method__sig: 'dppppp',
  _emval_coro_make_promise__sig: 'ppp',
  _emval_coro_suspend__sig: 'vpp',
  _emval_decref__sig: 'vp',
  _emval_delete__sig: 'ipp',
  _emval_equals__sig: 'ipp',
  _emval_get_global__sig: 'pp',
  _emval_get_method_caller__sig: 'pipi',
  _emval_get_module_property__sig: 'pp',
  _emval_get_property__sig: 'ppp',
  _emval_greater_than__sig: 'ipp',
  _emval_in__sig: 'ipp',
  _emval_incref__sig: 'vp',
  _emval_instanceof__sig: 'ipp',
  _emval_is_number__sig: 'ip',
  _emval_is_string__sig: 'ip',
  _emval_iter_begin__sig: 'pp',
  _emval_iter_next__sig: 'pp',
  _emval_less_than__sig: 'ipp',
  _emval_new_array__sig: 'p',
  _emval_new_array_from_memory_view__sig: 'pp',
  _emval_new_cstring__sig: 'pp',
  _emval_new_object__sig: 'p',
  _emval_new_u16string__sig: 'pp',
  _emval_new_u8string__sig: 'pp',
  _emval_not__sig: 'ip',
  _emval_register_symbol__sig: 'vp',
  _emval_run_destructors__sig: 'vp',
  _emval_set_property__sig: 'vppp',
  _emval_strictly_equals__sig: 'ipp',
  _emval_take_value__sig: 'ppp',
  _emval_throw__sig: 'ip',
  _emval_typeof__sig: 'pp',
  _gmtime_js__sig: 'vjp',
  _localtime_js__sig: 'vjp',
  _mktime_js__sig: 'jp',
  _mmap_js__sig: 'ipiiijpp',
  _msync_js__sig: 'ippiiij',
  _munmap_js__sig: 'ippiiij',
  _setitimer_js__sig: 'iid',
  _timegm_js__sig: 'jp',
  _tzset_js__sig: 'vppp',
  _wasmfs_copy_preloaded_file_data__sig: 'vip',
  _wasmfs_create_fetch_backend_js__sig: 'vp',
  _wasmfs_create_js_file_backend_js__sig: 'vp',
  _wasmfs_get_num_preloaded_dirs__sig: 'i',
  _wasmfs_get_num_preloaded_files__sig: 'i',
  _wasmfs_get_preloaded_child_path__sig: 'vip',
  _wasmfs_get_preloaded_file_mode__sig: 'ii',
  _wasmfs_get_preloaded_file_size__sig: 'pi',
  _wasmfs_get_preloaded_parent_path__sig: 'vip',
  _wasmfs_get_preloaded_path_name__sig: 'vip',
  _wasmfs_jsimpl_alloc_file__sig: 'vpp',
  _wasmfs_jsimpl_async_alloc_file__sig: 'vppp',
  _wasmfs_jsimpl_async_free_file__sig: 'vppp',
  _wasmfs_jsimpl_async_get_size__sig: 'vpppp',
  _wasmfs_jsimpl_async_read__sig: 'vpppppjp',
  _wasmfs_jsimpl_async_write__sig: 'vpppppjp',
  _wasmfs_jsimpl_free_file__sig: 'vpp',
  _wasmfs_jsimpl_get_size__sig: 'ipp',
  _wasmfs_jsimpl_read__sig: 'ippppj',
  _wasmfs_jsimpl_write__sig: 'ippppj',
  _wasmfs_node_close__sig: 'ii',
  _wasmfs_node_fstat_size__sig: 'iip',
  _wasmfs_node_get_mode__sig: 'ipp',
  _wasmfs_node_insert_directory__sig: 'ipi',
  _wasmfs_node_insert_file__sig: 'ipi',
  _wasmfs_node_open__sig: 'ipp',
  _wasmfs_node_read__sig: 'iipiip',
  _wasmfs_node_readdir__sig: 'ipp',
  _wasmfs_node_rmdir__sig: 'ip',
  _wasmfs_node_stat_size__sig: 'ipp',
  _wasmfs_node_unlink__sig: 'ip',
  _wasmfs_node_write__sig: 'iipiip',
  _wasmfs_opfs_close_access__sig: 'vpip',
  _wasmfs_opfs_close_blob__sig: 'vi',
  _wasmfs_opfs_flush_access__sig: 'vpip',
  _wasmfs_opfs_free_directory__sig: 'vi',
  _wasmfs_opfs_free_file__sig: 'vi',
  _wasmfs_opfs_get_child__sig: 'vpippp',
  _wasmfs_opfs_get_entries__sig: 'vpipp',
  _wasmfs_opfs_get_size_access__sig: 'vpip',
  _wasmfs_opfs_get_size_blob__sig: 'ii',
  _wasmfs_opfs_get_size_file__sig: 'vpip',
  _wasmfs_opfs_init_root_directory__sig: 'vp',
  _wasmfs_opfs_insert_directory__sig: 'vpipp',
  _wasmfs_opfs_insert_file__sig: 'vpipp',
  _wasmfs_opfs_move_file__sig: 'vpiipp',
  _wasmfs_opfs_open_access__sig: 'vpip',
  _wasmfs_opfs_open_blob__sig: 'vpip',
  _wasmfs_opfs_read_access__sig: 'iipii',
  _wasmfs_opfs_read_blob__sig: 'ipipiip',
  _wasmfs_opfs_remove_child__sig: 'vpipp',
  _wasmfs_opfs_set_size_access__sig: 'vpijp',
  _wasmfs_opfs_set_size_file__sig: 'vpijp',
  _wasmfs_opfs_write_access__sig: 'iipii',
  _wasmfs_stdin_get_char__sig: 'i',
  _wasmfs_thread_utils_heartbeat__sig: 'vp',
  abort__sig: 'v',
  alBuffer3f__sig: 'viifff',
  alBuffer3i__sig: 'viiiii',
  alBufferData__sig: 'viipii',
  alBufferf__sig: 'viif',
  alBufferfv__sig: 'viip',
  alBufferi__sig: 'viii',
  alBufferiv__sig: 'viip',
  alDeleteBuffers__sig: 'vip',
  alDeleteSources__sig: 'vip',
  alDisable__sig: 'vi',
  alDistanceModel__sig: 'vi',
  alDopplerFactor__sig: 'vf',
  alDopplerVelocity__sig: 'vf',
  alEnable__sig: 'vi',
  alGenBuffers__sig: 'vip',
  alGenSources__sig: 'vip',
  alGetBoolean__sig: 'ii',
  alGetBooleanv__sig: 'vip',
  alGetBuffer3f__sig: 'viippp',
  alGetBuffer3i__sig: 'viippp',
  alGetBufferf__sig: 'viip',
  alGetBufferfv__sig: 'viip',
  alGetBufferi__sig: 'viip',
  alGetBufferiv__sig: 'viip',
  alGetDouble__sig: 'di',
  alGetDoublev__sig: 'vip',
  alGetEnumValue__sig: 'ip',
  alGetError__sig: 'i',
  alGetFloat__sig: 'fi',
  alGetFloatv__sig: 'vip',
  alGetInteger__sig: 'ii',
  alGetIntegerv__sig: 'vip',
  alGetListener3f__sig: 'vippp',
  alGetListener3i__sig: 'vippp',
  alGetListenerf__sig: 'vip',
  alGetListenerfv__sig: 'vip',
  alGetListeneri__sig: 'vip',
  alGetListeneriv__sig: 'vip',
  alGetSource3f__sig: 'viippp',
  alGetSource3i__sig: 'viippp',
  alGetSourcef__sig: 'viip',
  alGetSourcefv__sig: 'viip',
  alGetSourcei__sig: 'viip',
  alGetSourceiv__sig: 'viip',
  alGetString__sig: 'pi',
  alIsBuffer__sig: 'ii',
  alIsEnabled__sig: 'ii',
  alIsExtensionPresent__sig: 'ip',
  alIsSource__sig: 'ii',
  alListener3f__sig: 'vifff',
  alListener3i__sig: 'viiii',
  alListenerf__sig: 'vif',
  alListenerfv__sig: 'vip',
  alListeneri__sig: 'vii',
  alListeneriv__sig: 'vip',
  alSource3f__sig: 'viifff',
  alSource3i__sig: 'viiiii',
  alSourcePause__sig: 'vi',
  alSourcePausev__sig: 'vip',
  alSourcePlay__sig: 'vi',
  alSourcePlayv__sig: 'vip',
  alSourceQueueBuffers__sig: 'viip',
  alSourceRewind__sig: 'vi',
  alSourceRewindv__sig: 'vip',
  alSourceStop__sig: 'vi',
  alSourceStopv__sig: 'vip',
  alSourceUnqueueBuffers__sig: 'viip',
  alSourcef__sig: 'viif',
  alSourcefv__sig: 'viip',
  alSourcei__sig: 'viii',
  alSourceiv__sig: 'viip',
  alSpeedOfSound__sig: 'vf',
  alcCaptureCloseDevice__sig: 'ip',
  alcCaptureOpenDevice__sig: 'ppiii',
  alcCaptureSamples__sig: 'vppi',
  alcCaptureStart__sig: 'vp',
  alcCaptureStop__sig: 'vp',
  alcCloseDevice__sig: 'ip',
  alcCreateContext__sig: 'ppp',
  alcDestroyContext__sig: 'vp',
  alcGetContextsDevice__sig: 'pp',
  alcGetCurrentContext__sig: 'p',
  alcGetEnumValue__sig: 'ipp',
  alcGetError__sig: 'ip',
  alcGetIntegerv__sig: 'vpiip',
  alcGetString__sig: 'ppi',
  alcIsExtensionPresent__sig: 'ipp',
  alcMakeContextCurrent__sig: 'ip',
  alcOpenDevice__sig: 'pp',
  alcProcessContext__sig: 'vp',
  alcSuspendContext__sig: 'vp',
  args_get__sig: 'ipp',
  args_sizes_get__sig: 'ipp',
  boxColor__sig: 'ipiiiii',
  boxRGBA__sig: 'ipiiiiiiii',
  clock_res_get__sig: 'iip',
  clock_time_get__sig: 'iijp',
  eglBindAPI__sig: 'ii',
  eglChooseConfig__sig: 'ipppip',
  eglCreateContext__sig: 'ppppp',
  eglCreateWindowSurface__sig: 'pppip',
  eglDestroyContext__sig: 'ipp',
  eglDestroySurface__sig: 'ipp',
  eglGetConfigAttrib__sig: 'ippip',
  eglGetConfigs__sig: 'ippip',
  eglGetCurrentContext__sig: 'p',
  eglGetCurrentDisplay__sig: 'p',
  eglGetCurrentSurface__sig: 'pi',
  eglGetDisplay__sig: 'pp',
  eglGetError__sig: 'i',
  eglGetProcAddress__sig: 'pp',
  eglInitialize__sig: 'ippp',
  eglMakeCurrent__sig: 'ipppp',
  eglQueryAPI__sig: 'i',
  eglQueryContext__sig: 'ippip',
  eglQueryString__sig: 'ppi',
  eglQuerySurface__sig: 'ippip',
  eglReleaseThread__sig: 'i',
  eglSwapBuffers__sig: 'ipp',
  eglSwapInterval__sig: 'ipi',
  eglTerminate__sig: 'ip',
  eglWaitClient__sig: 'i',
  eglWaitGL__sig: 'i',
  eglWaitNative__sig: 'ii',
  ellipseColor__sig: 'ipiiiii',
  ellipseRGBA__sig: 'ipiiiiiiii',
  emscripten_SDL_SetEventHandler__sig: 'vpp',
  emscripten_asm_const_async_on_main_thread__sig: 'vppp',
  emscripten_asm_const_double__sig: 'dppp',
  emscripten_asm_const_double_sync_on_main_thread__sig: 'dppp',
  emscripten_asm_const_int__sig: 'ippp',
  emscripten_asm_const_int_sync_on_main_thread__sig: 'ippp',
  emscripten_asm_const_ptr__sig: 'pppp',
  emscripten_asm_const_ptr_sync_on_main_thread__sig: 'pppp',
  emscripten_async_call__sig: 'vppi',
  emscripten_async_load_script__sig: 'vppp',
  emscripten_async_run_script__sig: 'vpi',
  emscripten_async_wget__sig: 'vpppp',
  emscripten_async_wget2__sig: 'ipppppppp',
  emscripten_async_wget2_abort__sig: 'vi',
  emscripten_async_wget2_data__sig: 'ippppippp',
  emscripten_async_wget_data__sig: 'vpppp',
  emscripten_atomic_cancel_all_wait_asyncs__sig: 'i',
  emscripten_atomic_cancel_all_wait_asyncs_at_address__sig: 'ip',
  emscripten_atomic_cancel_wait_async__sig: 'ii',
  emscripten_atomic_wait_async__sig: 'ipippd',
  emscripten_atomics_is_lock_free__sig: 'ii',
  emscripten_audio_context_state__sig: 'ii',
  emscripten_audio_worklet_post_function_sig__sig: 'vippp',
  emscripten_audio_worklet_post_function_v__sig: 'vip',
  emscripten_audio_worklet_post_function_vd__sig: 'vipd',
  emscripten_audio_worklet_post_function_vdd__sig: 'vipdd',
  emscripten_audio_worklet_post_function_vddd__sig: 'vipddd',
  emscripten_audio_worklet_post_function_vi__sig: 'vipi',
  emscripten_audio_worklet_post_function_vii__sig: 'vipii',
  emscripten_audio_worklet_post_function_viii__sig: 'vipiii',
  emscripten_call_worker__sig: 'vippipp',
  emscripten_cancel_animation_frame__sig: 'vi',
  emscripten_cancel_main_loop__sig: 'v',
  emscripten_check_blocking_allowed__sig: 'v',
  emscripten_clear_immediate__sig: 'vi',
  emscripten_clear_interval__sig: 'vi',
  emscripten_clear_timeout__sig: 'vi',
  emscripten_console_error__sig: 'vp',
  emscripten_console_log__sig: 'vp',
  emscripten_console_warn__sig: 'vp',
  emscripten_create_audio_context__sig: 'ip',
  emscripten_create_wasm_audio_worklet_node__sig: 'iipppp',
  emscripten_create_wasm_audio_worklet_processor_async__sig: 'vippp',
  emscripten_create_worker__sig: 'ip',
  emscripten_current_thread_is_audio_worklet__sig: 'i',
  emscripten_current_thread_is_wasm_worker__sig: 'i',
  emscripten_date_now__sig: 'd',
  emscripten_dbg__sig: 'vp',
  emscripten_dbgn__sig: 'vpp',
  emscripten_debugger__sig: 'v',
  emscripten_destroy_audio_context__sig: 'vi',
  emscripten_destroy_web_audio_node__sig: 'vi',
  emscripten_destroy_worker__sig: 'vi',
  emscripten_enter_soft_fullscreen__sig: 'ipp',
  emscripten_err__sig: 'vp',
  emscripten_errn__sig: 'vpp',
  emscripten_exit_fullscreen__sig: 'i',
  emscripten_exit_pointerlock__sig: 'i',
  emscripten_exit_soft_fullscreen__sig: 'i',
  emscripten_exit_with_live_runtime__sig: 'v',
  emscripten_fiber_swap__sig: 'vpp',
  emscripten_force_exit__sig: 'vi',
  emscripten_get_battery_status__sig: 'ip',
  emscripten_get_callstack__sig: 'iipi',
  emscripten_get_canvas_element_size__sig: 'ippp',
  emscripten_get_canvas_size__sig: 'vppp',
  emscripten_get_compiler_setting__sig: 'pp',
  emscripten_get_device_pixel_ratio__sig: 'd',
  emscripten_get_devicemotion_status__sig: 'ip',
  emscripten_get_deviceorientation_status__sig: 'ip',
  emscripten_get_element_css_size__sig: 'ippp',
  emscripten_get_exported_function__sig: 'pp',
  emscripten_get_fullscreen_status__sig: 'ip',
  emscripten_get_gamepad_status__sig: 'iip',
  emscripten_get_heap_max__sig: 'p',
  emscripten_get_main_loop_timing__sig: 'vpp',
  emscripten_get_module_name__sig: 'ppp',
  emscripten_get_mouse_status__sig: 'ip',
  emscripten_get_now__sig: 'd',
  emscripten_get_now_res__sig: 'd',
  emscripten_get_num_gamepads__sig: 'i',
  emscripten_get_orientation_status__sig: 'ip',
  emscripten_get_pointerlock_status__sig: 'ip',
  emscripten_get_preloaded_image_data__sig: 'pppp',
  emscripten_get_preloaded_image_data_from_FILE__sig: 'pppp',
  emscripten_get_screen_size__sig: 'vpp',
  emscripten_get_visibility_status__sig: 'ip',
  emscripten_get_window_title__sig: 'p',
  emscripten_get_worker_queue_size__sig: 'ii',
  emscripten_has_asyncify__sig: 'i',
  emscripten_has_threading_support__sig: 'i',
  emscripten_hide_mouse__sig: 'v',
  emscripten_html5_remove_all_event_listeners__sig: 'v',
  emscripten_idb_async_clear__sig: 'vpppp',
  emscripten_idb_async_delete__sig: 'vppppp',
  emscripten_idb_async_exists__sig: 'vppppp',
  emscripten_idb_async_load__sig: 'vppppp',
  emscripten_idb_async_store__sig: 'vpppippp',
  emscripten_idb_clear__sig: 'vpp',
  emscripten_idb_delete__sig: 'vppp',
  emscripten_idb_exists__sig: 'vpppp',
  emscripten_idb_free_blob__sig: 'vi',
  emscripten_idb_load__sig: 'vppppp',
  emscripten_idb_load_blob__sig: 'vpppp',
  emscripten_idb_read_from_blob__sig: 'viiip',
  emscripten_idb_store__sig: 'vpppip',
  emscripten_idb_store_blob__sig: 'vpppip',
  emscripten_is_webgl_context_lost__sig: 'ip',
  emscripten_lazy_load_code__sig: 'v',
  emscripten_lock_async_acquire__sig: 'vpppd',
  emscripten_lock_orientation__sig: 'ii',
  emscripten_log__sig: 'vipp',
  emscripten_math_acos__sig: 'dd',
  emscripten_math_acosh__sig: 'dd',
  emscripten_math_asin__sig: 'dd',
  emscripten_math_asinh__sig: 'dd',
  emscripten_math_atan__sig: 'dd',
  emscripten_math_atan2__sig: 'ddd',
  emscripten_math_atanh__sig: 'dd',
  emscripten_math_cbrt__sig: 'dd',
  emscripten_math_cos__sig: 'dd',
  emscripten_math_cosh__sig: 'dd',
  emscripten_math_exp__sig: 'dd',
  emscripten_math_expm1__sig: 'dd',
  emscripten_math_fmod__sig: 'ddd',
  emscripten_math_hypot__sig: 'dip',
  emscripten_math_log__sig: 'dd',
  emscripten_math_log10__sig: 'dd',
  emscripten_math_log1p__sig: 'dd',
  emscripten_math_log2__sig: 'dd',
  emscripten_math_pow__sig: 'ddd',
  emscripten_math_random__sig: 'd',
  emscripten_math_round__sig: 'dd',
  emscripten_math_sign__sig: 'dd',
  emscripten_math_sin__sig: 'dd',
  emscripten_math_sinh__sig: 'dd',
  emscripten_math_sqrt__sig: 'dd',
  emscripten_math_tan__sig: 'dd',
  emscripten_math_tanh__sig: 'dd',
  emscripten_memcpy_js__sig: 'vppp',
  emscripten_navigator_hardware_concurrency__sig: 'i',
  emscripten_notify_memory_growth__sig: 'vp',
  emscripten_num_logical_cores__sig: 'i',
  emscripten_out__sig: 'vp',
  emscripten_outn__sig: 'vpp',
  emscripten_pause_main_loop__sig: 'v',
  emscripten_pc_get_column__sig: 'ip',
  emscripten_pc_get_file__sig: 'pp',
  emscripten_pc_get_function__sig: 'pp',
  emscripten_pc_get_line__sig: 'ip',
  emscripten_performance_now__sig: 'd',
  emscripten_print_double__sig: 'idpi',
  emscripten_promise_all__sig: 'pppp',
  emscripten_promise_all_settled__sig: 'pppp',
  emscripten_promise_any__sig: 'pppp',
  emscripten_promise_await__sig: 'vpp',
  emscripten_promise_create__sig: 'p',
  emscripten_promise_destroy__sig: 'vp',
  emscripten_promise_race__sig: 'ppp',
  emscripten_promise_resolve__sig: 'vpip',
  emscripten_promise_then__sig: 'ppppp',
  emscripten_random__sig: 'f',
  emscripten_request_animation_frame__sig: 'ipp',
  emscripten_request_animation_frame_loop__sig: 'vpp',
  emscripten_request_fullscreen__sig: 'ipi',
  emscripten_request_fullscreen_strategy__sig: 'ipip',
  emscripten_request_pointerlock__sig: 'ipi',
  emscripten_resize_heap__sig: 'ip',
  emscripten_resume_audio_context_async__sig: 'vipp',
  emscripten_resume_audio_context_sync__sig: 'vi',
  emscripten_resume_main_loop__sig: 'v',
  emscripten_return_address__sig: 'pi',
  emscripten_run_preload_plugins__sig: 'ippp',
  emscripten_run_preload_plugins_data__sig: 'vpipppp',
  emscripten_run_script__sig: 'vp',
  emscripten_run_script_int__sig: 'ip',
  emscripten_run_script_string__sig: 'pp',
  emscripten_runtime_keepalive_check__sig: 'i',
  emscripten_runtime_keepalive_pop__sig: 'v',
  emscripten_runtime_keepalive_push__sig: 'v',
  emscripten_sample_gamepad_data__sig: 'i',
  emscripten_scan_registers__sig: 'vp',
  emscripten_semaphore_async_acquire__sig: 'vpippd',
  emscripten_set_batterychargingchange_callback_on_thread__sig: 'ippp',
  emscripten_set_batterylevelchange_callback_on_thread__sig: 'ippp',
  emscripten_set_beforeunload_callback_on_thread__sig: 'ippp',
  emscripten_set_blur_callback_on_thread__sig: 'ippipp',
  emscripten_set_canvas_element_size__sig: 'ipii',
  emscripten_set_canvas_size__sig: 'vii',
  emscripten_set_click_callback_on_thread__sig: 'ippipp',
  emscripten_set_dblclick_callback_on_thread__sig: 'ippipp',
  emscripten_set_devicemotion_callback_on_thread__sig: 'ipipp',
  emscripten_set_deviceorientation_callback_on_thread__sig: 'ipipp',
  emscripten_set_element_css_size__sig: 'ipdd',
  emscripten_set_focus_callback_on_thread__sig: 'ippipp',
  emscripten_set_focusin_callback_on_thread__sig: 'ippipp',
  emscripten_set_focusout_callback_on_thread__sig: 'ippipp',
  emscripten_set_fullscreenchange_callback_on_thread__sig: 'ippipp',
  emscripten_set_gamepadconnected_callback_on_thread__sig: 'ipipp',
  emscripten_set_gamepaddisconnected_callback_on_thread__sig: 'ipipp',
  emscripten_set_immediate__sig: 'ipp',
  emscripten_set_immediate_loop__sig: 'vpp',
  emscripten_set_interval__sig: 'ipdp',
  emscripten_set_keydown_callback_on_thread__sig: 'ippipp',
  emscripten_set_keypress_callback_on_thread__sig: 'ippipp',
  emscripten_set_keyup_callback_on_thread__sig: 'ippipp',
  emscripten_set_main_loop__sig: 'vpii',
  emscripten_set_main_loop_arg__sig: 'vppii',
  emscripten_set_main_loop_expected_blockers__sig: 'vi',
  emscripten_set_main_loop_timing__sig: 'iii',
  emscripten_set_mousedown_callback_on_thread__sig: 'ippipp',
  emscripten_set_mouseenter_callback_on_thread__sig: 'ippipp',
  emscripten_set_mouseleave_callback_on_thread__sig: 'ippipp',
  emscripten_set_mousemove_callback_on_thread__sig: 'ippipp',
  emscripten_set_mouseout_callback_on_thread__sig: 'ippipp',
  emscripten_set_mouseover_callback_on_thread__sig: 'ippipp',
  emscripten_set_mouseup_callback_on_thread__sig: 'ippipp',
  emscripten_set_orientationchange_callback_on_thread__sig: 'ipipp',
  emscripten_set_pointerlockchange_callback_on_thread__sig: 'ippipp',
  emscripten_set_pointerlockerror_callback_on_thread__sig: 'ippipp',
  emscripten_set_resize_callback_on_thread__sig: 'ippipp',
  emscripten_set_scroll_callback_on_thread__sig: 'ippipp',
  emscripten_set_socket_close_callback__sig: 'vpp',
  emscripten_set_socket_connection_callback__sig: 'vpp',
  emscripten_set_socket_error_callback__sig: 'vpp',
  emscripten_set_socket_listen_callback__sig: 'vpp',
  emscripten_set_socket_message_callback__sig: 'vpp',
  emscripten_set_socket_open_callback__sig: 'vpp',
  emscripten_set_timeout__sig: 'ipdp',
  emscripten_set_timeout_loop__sig: 'vpdp',
  emscripten_set_touchcancel_callback_on_thread__sig: 'ippipp',
  emscripten_set_touchend_callback_on_thread__sig: 'ippipp',
  emscripten_set_touchmove_callback_on_thread__sig: 'ippipp',
  emscripten_set_touchstart_callback_on_thread__sig: 'ippipp',
  emscripten_set_visibilitychange_callback_on_thread__sig: 'ipipp',
  emscripten_set_webglcontextlost_callback_on_thread__sig: 'ippipp',
  emscripten_set_webglcontextrestored_callback_on_thread__sig: 'ippipp',
  emscripten_set_wheel_callback_on_thread__sig: 'ippipp',
  emscripten_set_window_title__sig: 'vp',
  emscripten_sleep__sig: 'vi',
  emscripten_stack_snapshot__sig: 'p',
  emscripten_stack_unwind_buffer__sig: 'ippi',
  emscripten_start_fetch__sig: 'vp',
  emscripten_start_wasm_audio_worklet_thread_async__sig: 'vipipp',
  emscripten_supports_offscreencanvas__sig: 'i',
  emscripten_terminate_all_wasm_workers__sig: 'v',
  emscripten_terminate_wasm_worker__sig: 'vi',
  emscripten_throw_number__sig: 'vd',
  emscripten_throw_string__sig: 'vp',
  emscripten_trace_annotate_address_type__sig: 'vpp',
  emscripten_trace_associate_storage_size__sig: 'vpi',
  emscripten_trace_close__sig: 'v',
  emscripten_trace_configure__sig: 'vpp',
  emscripten_trace_configure_for_google_wtf__sig: 'v',
  emscripten_trace_configure_for_test__sig: 'v',
  emscripten_trace_enter_context__sig: 'vp',
  emscripten_trace_exit_context__sig: 'v',
  emscripten_trace_log_message__sig: 'vpp',
  emscripten_trace_mark__sig: 'vp',
  emscripten_trace_record_allocation__sig: 'vpi',
  emscripten_trace_record_frame_end__sig: 'v',
  emscripten_trace_record_frame_start__sig: 'v',
  emscripten_trace_record_free__sig: 'vp',
  emscripten_trace_record_reallocation__sig: 'vppi',
  emscripten_trace_report_error__sig: 'vp',
  emscripten_trace_report_memory_layout__sig: 'v',
  emscripten_trace_report_off_heap_data__sig: 'v',
  emscripten_trace_set_enabled__sig: 'vi',
  emscripten_trace_set_session_username__sig: 'vp',
  emscripten_trace_task_associate_data__sig: 'vpp',
  emscripten_trace_task_end__sig: 'v',
  emscripten_trace_task_resume__sig: 'vip',
  emscripten_trace_task_start__sig: 'vip',
  emscripten_trace_task_suspend__sig: 'vp',
  emscripten_unlock_orientation__sig: 'i',
  emscripten_unwind_to_js_event_loop__sig: 'v',
  emscripten_vibrate__sig: 'ii',
  emscripten_vibrate_pattern__sig: 'ipi',
  emscripten_wasm_worker_post_function_sig__sig: 'vippp',
  emscripten_wasm_worker_post_function_v__sig: 'vip',
  emscripten_wasm_worker_post_function_vd__sig: 'vipd',
  emscripten_wasm_worker_post_function_vdd__sig: 'vipdd',
  emscripten_wasm_worker_post_function_vddd__sig: 'vipddd',
  emscripten_wasm_worker_post_function_vi__sig: 'vipi',
  emscripten_wasm_worker_post_function_vii__sig: 'vipii',
  emscripten_wasm_worker_post_function_viii__sig: 'vipiii',
  emscripten_wasm_worker_self_id__sig: 'i',
  emscripten_webgl1_get_proc_address__sig: 'pp',
  emscripten_webgl2_get_proc_address__sig: 'pp',
  emscripten_webgl_commit_frame__sig: 'i',
  emscripten_webgl_create_context__sig: 'ppp',
  emscripten_webgl_destroy_context__sig: 'ip',
  emscripten_webgl_do_commit_frame__sig: 'i',
  emscripten_webgl_do_create_context__sig: 'ppp',
  emscripten_webgl_do_get_current_context__sig: 'p',
  emscripten_webgl_enable_ANGLE_instanced_arrays__sig: 'ip',
  emscripten_webgl_enable_OES_vertex_array_object__sig: 'ip',
  emscripten_webgl_enable_WEBGL_draw_buffers__sig: 'ip',
  emscripten_webgl_enable_WEBGL_draw_instanced_base_vertex_base_instance__sig: 'ip',
  emscripten_webgl_enable_WEBGL_multi_draw__sig: 'ip',
  emscripten_webgl_enable_WEBGL_multi_draw_instanced_base_vertex_base_instance__sig: 'ip',
  emscripten_webgl_enable_extension__sig: 'ipp',
  emscripten_webgl_get_context_attributes__sig: 'ipp',
  emscripten_webgl_get_current_context__sig: 'p',
  emscripten_webgl_get_drawing_buffer_size__sig: 'ippp',
  emscripten_webgl_get_parameter_d__sig: 'di',
  emscripten_webgl_get_parameter_i64v__sig: 'vip',
  emscripten_webgl_get_parameter_o__sig: 'ii',
  emscripten_webgl_get_parameter_utf8__sig: 'pi',
  emscripten_webgl_get_parameter_v__sig: 'iipii',
  emscripten_webgl_get_proc_address__sig: 'pp',
  emscripten_webgl_get_program_info_log_utf8__sig: 'pi',
  emscripten_webgl_get_program_parameter_d__sig: 'dii',
  emscripten_webgl_get_shader_info_log_utf8__sig: 'pi',
  emscripten_webgl_get_shader_parameter_d__sig: 'dii',
  emscripten_webgl_get_shader_source_utf8__sig: 'pi',
  emscripten_webgl_get_supported_extensions__sig: 'p',
  emscripten_webgl_get_uniform_d__sig: 'dii',
  emscripten_webgl_get_uniform_v__sig: 'iiipii',
  emscripten_webgl_get_vertex_attrib_d__sig: 'dii',
  emscripten_webgl_get_vertex_attrib_o__sig: 'iii',
  emscripten_webgl_get_vertex_attrib_v__sig: 'iiipii',
  emscripten_webgl_init_context_attributes__sig: 'vp',
  emscripten_webgl_make_context_current__sig: 'ip',
  emscripten_webgpu_export_bind_group__sig: 'ip',
  emscripten_webgpu_export_bind_group_layout__sig: 'ip',
  emscripten_webgpu_export_buffer__sig: 'ip',
  emscripten_webgpu_export_command_buffer__sig: 'ip',
  emscripten_webgpu_export_command_encoder__sig: 'ip',
  emscripten_webgpu_export_compute_pass_encoder__sig: 'ip',
  emscripten_webgpu_export_compute_pipeline__sig: 'ip',
  emscripten_webgpu_export_device__sig: 'ip',
  emscripten_webgpu_export_pipeline_layout__sig: 'ip',
  emscripten_webgpu_export_query_set__sig: 'ip',
  emscripten_webgpu_export_queue__sig: 'ip',
  emscripten_webgpu_export_render_bundle__sig: 'ip',
  emscripten_webgpu_export_render_bundle_encoder__sig: 'ip',
  emscripten_webgpu_export_render_pass_encoder__sig: 'ip',
  emscripten_webgpu_export_render_pipeline__sig: 'ip',
  emscripten_webgpu_export_sampler__sig: 'ip',
  emscripten_webgpu_export_shader_module__sig: 'ip',
  emscripten_webgpu_export_surface__sig: 'ip',
  emscripten_webgpu_export_swap_chain__sig: 'ip',
  emscripten_webgpu_export_texture__sig: 'ip',
  emscripten_webgpu_export_texture_view__sig: 'ip',
  emscripten_webgpu_get_device__sig: 'p',
  emscripten_webgpu_import_bind_group__sig: 'pi',
  emscripten_webgpu_import_bind_group_layout__sig: 'pi',
  emscripten_webgpu_import_buffer__sig: 'pi',
  emscripten_webgpu_import_command_buffer__sig: 'pi',
  emscripten_webgpu_import_command_encoder__sig: 'pi',
  emscripten_webgpu_import_compute_pass_encoder__sig: 'pi',
  emscripten_webgpu_import_compute_pipeline__sig: 'pi',
  emscripten_webgpu_import_device__sig: 'pi',
  emscripten_webgpu_import_pipeline_layout__sig: 'pi',
  emscripten_webgpu_import_query_set__sig: 'pi',
  emscripten_webgpu_import_queue__sig: 'pi',
  emscripten_webgpu_import_render_bundle__sig: 'pi',
  emscripten_webgpu_import_render_bundle_encoder__sig: 'pi',
  emscripten_webgpu_import_render_pass_encoder__sig: 'pi',
  emscripten_webgpu_import_render_pipeline__sig: 'pi',
  emscripten_webgpu_import_sampler__sig: 'pi',
  emscripten_webgpu_import_shader_module__sig: 'pi',
  emscripten_webgpu_import_surface__sig: 'pi',
  emscripten_webgpu_import_swap_chain__sig: 'pi',
  emscripten_webgpu_import_texture__sig: 'pi',
  emscripten_webgpu_import_texture_view__sig: 'pi',
  emscripten_webgpu_release_js_handle__sig: 'vi',
  emscripten_websocket_close__sig: 'iiip',
  emscripten_websocket_deinitialize__sig: 'v',
  emscripten_websocket_delete__sig: 'ii',
  emscripten_websocket_get_buffered_amount__sig: 'iip',
  emscripten_websocket_get_extensions__sig: 'iipi',
  emscripten_websocket_get_extensions_length__sig: 'iip',
  emscripten_websocket_get_protocol__sig: 'iipi',
  emscripten_websocket_get_protocol_length__sig: 'iip',
  emscripten_websocket_get_ready_state__sig: 'iip',
  emscripten_websocket_get_url__sig: 'iipi',
  emscripten_websocket_get_url_length__sig: 'iip',
  emscripten_websocket_is_supported__sig: 'i',
  emscripten_websocket_new__sig: 'ip',
  emscripten_websocket_send_binary__sig: 'iipi',
  emscripten_websocket_send_utf8_text__sig: 'iip',
  emscripten_websocket_set_onclose_callback_on_thread__sig: 'iippp',
  emscripten_websocket_set_onerror_callback_on_thread__sig: 'iippp',
  emscripten_websocket_set_onmessage_callback_on_thread__sig: 'iippp',
  emscripten_websocket_set_onopen_callback_on_thread__sig: 'iippp',
  emscripten_wget__sig: 'ipp',
  emscripten_wget_data__sig: 'vpppp',
  emscripten_worker_respond__sig: 'vpi',
  emscripten_worker_respond_provisionally__sig: 'vpi',
  endprotoent__sig: 'v',
  environ_get__sig: 'ipp',
  environ_sizes_get__sig: 'ipp',
  exit__sig: 'vi',
  fd_close__sig: 'ii',
  fd_fdstat_get__sig: 'iip',
  fd_pread__sig: 'iippjp',
  fd_pwrite__sig: 'iippjp',
  fd_read__sig: 'iippp',
  fd_seek__sig: 'iijip',
  fd_sync__sig: 'ii',
  fd_write__sig: 'iippp',
  filledEllipseColor__sig: 'ipiiiii',
  filledEllipseRGBA__sig: 'ipiiiiiiii',
  getaddrinfo__sig: 'ipppp',
  getentropy__sig: 'ipp',
  gethostbyaddr__sig: 'ppii',
  gethostbyname__sig: 'pp',
  gethostbyname_r__sig: 'ipppppp',
  getnameinfo__sig: 'ipipipii',
  getprotobyname__sig: 'pp',
  getprotobynumber__sig: 'pi',
  getprotoent__sig: 'p',
  glActiveTexture__sig: 'vi',
  glAlphaFunc__sig: 'vif',
  glAttachShader__sig: 'vii',
  glBegin__sig: 'vi',
  glBeginQuery__sig: 'vii',
  glBeginTransformFeedback__sig: 'vi',
  glBindAttribLocation__sig: 'viip',
  glBindBuffer__sig: 'vii',
  glBindBufferBase__sig: 'viii',
  glBindBufferRange__sig: 'viiipp',
  glBindFramebuffer__sig: 'vii',
  glBindFramebufferOES__sig: 'vii',
  glBindProgram__sig: 'vii',
  glBindRenderbuffer__sig: 'vii',
  glBindRenderbufferOES__sig: 'vii',
  glBindSampler__sig: 'vii',
  glBindTexture__sig: 'vii',
  glBindTransformFeedback__sig: 'vii',
  glBindVertexArray__sig: 'vi',
  glBindVertexArrayOES__sig: 'vi',
  glBlendColor__sig: 'vffff',
  glBlendEquation__sig: 'vi',
  glBlendEquationSeparate__sig: 'vii',
  glBlendFunc__sig: 'vii',
  glBlendFuncSeparate__sig: 'viiii',
  glBlitFramebuffer__sig: 'viiiiiiiiii',
  glBufferData__sig: 'vippi',
  glBufferSubData__sig: 'vippp',
  glCheckFramebufferStatus__sig: 'ii',
  glCheckFramebufferStatusOES__sig: 'ii',
  glClear__sig: 'vi',
  glClearBufferfi__sig: 'viifi',
  glClearBufferfv__sig: 'viip',
  glClearBufferiv__sig: 'viip',
  glClearBufferuiv__sig: 'viip',
  glClearColor__sig: 'vffff',
  glClearDepth__sig: 'vd',
  glClearDepthf__sig: 'vf',
  glClearStencil__sig: 'vi',
  glClientActiveTexture__sig: 'vi',
  glClientWaitSync__sig: 'ipij',
  glClipPlane__sig: 'vip',
  glColor3d__sig: 'vddd',
  glColor3f__sig: 'vfff',
  glColor3fv__sig: 'vp',
  glColor3ub__sig: 'viii',
  glColor3ubv__sig: 'vp',
  glColor3ui__sig: 'viii',
  glColor3uiv__sig: 'vp',
  glColor3us__sig: 'viii',
  glColor3usv__sig: 'vp',
  glColor4d__sig: 'vdddd',
  glColor4f__sig: 'vffff',
  glColor4fv__sig: 'vp',
  glColor4ub__sig: 'viiii',
  glColor4ubv__sig: 'vp',
  glColor4ui__sig: 'viiii',
  glColor4us__sig: 'viiii',
  glColorMask__sig: 'viiii',
  glColorPointer__sig: 'viiip',
  glCompileShader__sig: 'vi',
  glCompressedTexImage2D__sig: 'viiiiiiip',
  glCompressedTexImage3D__sig: 'viiiiiiiip',
  glCompressedTexSubImage2D__sig: 'viiiiiiiip',
  glCompressedTexSubImage3D__sig: 'viiiiiiiiiip',
  glCopyBufferSubData__sig: 'viippp',
  glCopyTexImage2D__sig: 'viiiiiiii',
  glCopyTexSubImage2D__sig: 'viiiiiiii',
  glCopyTexSubImage3D__sig: 'viiiiiiiii',
  glCreateProgram__sig: 'i',
  glCreateShader__sig: 'ii',
  glCullFace__sig: 'vi',
  glDeleteBuffers__sig: 'vip',
  glDeleteFramebuffers__sig: 'vip',
  glDeleteFramebuffersOES__sig: 'vip',
  glDeleteObject__sig: 'vi',
  glDeleteProgram__sig: 'vi',
  glDeleteQueries__sig: 'vip',
  glDeleteRenderbuffers__sig: 'vip',
  glDeleteRenderbuffersOES__sig: 'vip',
  glDeleteSamplers__sig: 'vip',
  glDeleteShader__sig: 'vi',
  glDeleteSync__sig: 'vp',
  glDeleteTextures__sig: 'vip',
  glDeleteTransformFeedbacks__sig: 'vip',
  glDeleteVertexArrays__sig: 'vip',
  glDeleteVertexArraysOES__sig: 'vip',
  glDepthFunc__sig: 'vi',
  glDepthMask__sig: 'vi',
  glDepthRange__sig: 'vdd',
  glDepthRangef__sig: 'vff',
  glDetachShader__sig: 'vii',
  glDisable__sig: 'vi',
  glDisableClientState__sig: 'vi',
  glDisableVertexAttribArray__sig: 'vi',
  glDrawArrays__sig: 'viii',
  glDrawArraysInstanced__sig: 'viiii',
  glDrawArraysInstancedBaseInstance__sig: 'viiiii',
  glDrawBuffer__sig: 'vi',
  glDrawBuffers__sig: 'vip',
  glDrawElements__sig: 'viiip',
  glDrawElementsInstanced__sig: 'viiipi',
  glDrawRangeElements__sig: 'viiiiip',
  glEnable__sig: 'vi',
  glEnableClientState__sig: 'vi',
  glEnableVertexAttribArray__sig: 'vi',
  glEnd__sig: 'v',
  glEndQuery__sig: 'vi',
  glEndTransformFeedback__sig: 'v',
  glFenceSync__sig: 'pii',
  glFinish__sig: 'v',
  glFlush__sig: 'v',
  glFlushMappedBufferRange__sig: 'vipp',
  glFogf__sig: 'vif',
  glFogfv__sig: 'vip',
  glFogi__sig: 'vii',
  glFogiv__sig: 'vip',
  glFogx__sig: 'vii',
  glFogxv__sig: 'vip',
  glFramebufferRenderbuffer__sig: 'viiii',
  glFramebufferRenderbufferOES__sig: 'viiii',
  glFramebufferTexture2D__sig: 'viiiii',
  glFramebufferTexture2DOES__sig: 'viiiii',
  glFramebufferTextureLayer__sig: 'viiiii',
  glFrontFace__sig: 'vi',
  glFrustum__sig: 'vdddddd',
  glFrustumf__sig: 'vffffff',
  glGenBuffers__sig: 'vip',
  glGenFramebuffers__sig: 'vip',
  glGenFramebuffersOES__sig: 'vip',
  glGenQueries__sig: 'vip',
  glGenRenderbuffers__sig: 'vip',
  glGenRenderbuffersOES__sig: 'vip',
  glGenSamplers__sig: 'vip',
  glGenTextures__sig: 'vip',
  glGenTransformFeedbacks__sig: 'vip',
  glGenVertexArrays__sig: 'vip',
  glGenVertexArraysOES__sig: 'vip',
  glGenerateMipmap__sig: 'vi',
  glGetActiveAttrib__sig: 'viiipppp',
  glGetActiveUniform__sig: 'viiipppp',
  glGetActiveUniformBlockName__sig: 'viiipp',
  glGetActiveUniformBlockiv__sig: 'viiip',
  glGetActiveUniformsiv__sig: 'viipip',
  glGetAttachedShaders__sig: 'viipp',
  glGetAttribLocation__sig: 'iip',
  glGetBooleanv__sig: 'vip',
  glGetBufferParameteri64v__sig: 'viip',
  glGetBufferParameteriv__sig: 'viip',
  glGetBufferPointerv__sig: 'viip',
  glGetBufferSubData__sig: 'vippp',
  glGetError__sig: 'i',
  glGetFloatv__sig: 'vip',
  glGetFragDataLocation__sig: 'iip',
  glGetFramebufferAttachmentParameteriv__sig: 'viiip',
  glGetInfoLog__sig: 'viipp',
  glGetInteger64i_v__sig: 'viip',
  glGetInteger64v__sig: 'vip',
  glGetIntegeri_v__sig: 'viip',
  glGetIntegerv__sig: 'vip',
  glGetInternalformativ__sig: 'viiiip',
  glGetObjectParameteriv__sig: 'viip',
  glGetPointerv__sig: 'vip',
  glGetProgramBinary__sig: 'viippp',
  glGetProgramInfoLog__sig: 'viipp',
  glGetProgramiv__sig: 'viip',
  glGetQueryObjectuiv__sig: 'viip',
  glGetQueryiv__sig: 'viip',
  glGetRenderbufferParameteriv__sig: 'viip',
  glGetRenderbufferParameterivOES__sig: 'viip',
  glGetSamplerParameterfv__sig: 'viip',
  glGetSamplerParameteriv__sig: 'viip',
  glGetShaderInfoLog__sig: 'viipp',
  glGetShaderPrecisionFormat__sig: 'viipp',
  glGetShaderSource__sig: 'viipp',
  glGetShaderiv__sig: 'viip',
  glGetString__sig: 'pi',
  glGetStringi__sig: 'pii',
  glGetSynciv__sig: 'vpiipp',
  glGetTexEnvfv__sig: 'viip',
  glGetTexEnviv__sig: 'viip',
  glGetTexLevelParameteriv__sig: 'viiip',
  glGetTexParameterfv__sig: 'viip',
  glGetTexParameteriv__sig: 'viip',
  glGetTransformFeedbackVarying__sig: 'viiipppp',
  glGetUniformBlockIndex__sig: 'iip',
  glGetUniformIndices__sig: 'viipp',
  glGetUniformLocation__sig: 'iip',
  glGetUniformfv__sig: 'viip',
  glGetUniformiv__sig: 'viip',
  glGetUniformuiv__sig: 'viip',
  glGetVertexAttribIiv__sig: 'viip',
  glGetVertexAttribIuiv__sig: 'viip',
  glGetVertexAttribPointerv__sig: 'viip',
  glGetVertexAttribfv__sig: 'viip',
  glGetVertexAttribiv__sig: 'viip',
  glHint__sig: 'vii',
  glInvalidateFramebuffer__sig: 'viip',
  glInvalidateSubFramebuffer__sig: 'viipiiii',
  glIsBuffer__sig: 'ii',
  glIsEnabled__sig: 'ii',
  glIsFramebuffer__sig: 'ii',
  glIsProgram__sig: 'ii',
  glIsQuery__sig: 'ii',
  glIsRenderbuffer__sig: 'ii',
  glIsSampler__sig: 'ii',
  glIsShader__sig: 'ii',
  glIsSync__sig: 'ip',
  glIsTexture__sig: 'ii',
  glIsTransformFeedback__sig: 'ii',
  glIsVertexArray__sig: 'ii',
  glIsVertexArrayOES__sig: 'ii',
  glLightModelf__sig: 'vif',
  glLightModelfv__sig: 'vip',
  glLightfv__sig: 'viip',
  glLineWidth__sig: 'vf',
  glLinkProgram__sig: 'vi',
  glLoadIdentity__sig: 'v',
  glLoadMatrixd__sig: 'vp',
  glLoadMatrixf__sig: 'vp',
  glLoadTransposeMatrixd__sig: 'vp',
  glLoadTransposeMatrixf__sig: 'vp',
  glMapBufferRange__sig: 'pippi',
  glMaterialfv__sig: 'viip',
  glMatrixMode__sig: 'vi',
  glMultMatrixd__sig: 'vp',
  glMultMatrixf__sig: 'vp',
  glMultTransposeMatrixd__sig: 'vp',
  glMultTransposeMatrixf__sig: 'vp',
  glMultiDrawArrays__sig: 'vippi',
  glMultiDrawElements__sig: 'vipipi',
  glNormal3f__sig: 'vfff',
  glNormal3fv__sig: 'vp',
  glNormalPointer__sig: 'viip',
  glOrtho__sig: 'vdddddd',
  glOrthof__sig: 'vffffff',
  glPauseTransformFeedback__sig: 'v',
  glPixelStorei__sig: 'vii',
  glPointSize__sig: 'vf',
  glPolygonMode__sig: 'vii',
  glPolygonOffset__sig: 'vff',
  glPopMatrix__sig: 'v',
  glProgramBinary__sig: 'viipi',
  glProgramParameteri__sig: 'viii',
  glPushMatrix__sig: 'v',
  glReadBuffer__sig: 'vi',
  glReadPixels__sig: 'viiiiiip',
  glReleaseShaderCompiler__sig: 'v',
  glRenderbufferStorage__sig: 'viiii',
  glRenderbufferStorageMultisample__sig: 'viiiii',
  glRenderbufferStorageOES__sig: 'viiii',
  glResumeTransformFeedback__sig: 'v',
  glRotated__sig: 'vdddd',
  glRotatef__sig: 'vffff',
  glSampleCoverage__sig: 'vfi',
  glSamplerParameterf__sig: 'viif',
  glSamplerParameterfv__sig: 'viip',
  glSamplerParameteri__sig: 'viii',
  glSamplerParameteriv__sig: 'viip',
  glScaled__sig: 'vddd',
  glScalef__sig: 'vfff',
  glScissor__sig: 'viiii',
  glShadeModel__sig: 'vi',
  glShaderBinary__sig: 'vipipi',
  glShaderSource__sig: 'viipp',
  glStencilFunc__sig: 'viii',
  glStencilFuncSeparate__sig: 'viiii',
  glStencilMask__sig: 'vi',
  glStencilMaskSeparate__sig: 'vii',
  glStencilOp__sig: 'viii',
  glStencilOpSeparate__sig: 'viiii',
  glTexCoord2f__sig: 'vff',
  glTexCoord2fv__sig: 'vp',
  glTexCoord2i__sig: 'vii',
  glTexCoord3f__sig: 'vfff',
  glTexCoord4f__sig: 'vffff',
  glTexCoordPointer__sig: 'viiip',
  glTexEnvf__sig: 'viif',
  glTexEnvfv__sig: 'viip',
  glTexEnvi__sig: 'viii',
  glTexGenfv__sig: 'viip',
  glTexGeni__sig: 'viii',
  glTexImage1D__sig: 'viiiiiiip',
  glTexImage2D__sig: 'viiiiiiiip',
  glTexImage3D__sig: 'viiiiiiiiip',
  glTexParameterf__sig: 'viif',
  glTexParameterfv__sig: 'viip',
  glTexParameteri__sig: 'viii',
  glTexParameteriv__sig: 'viip',
  glTexStorage2D__sig: 'viiiii',
  glTexStorage3D__sig: 'viiiiii',
  glTexSubImage2D__sig: 'viiiiiiiip',
  glTexSubImage3D__sig: 'viiiiiiiiiip',
  glTransformFeedbackVaryings__sig: 'viipi',
  glTranslated__sig: 'vddd',
  glTranslatef__sig: 'vfff',
  glUniform1f__sig: 'vif',
  glUniform1fv__sig: 'viip',
  glUniform1i__sig: 'vii',
  glUniform1iv__sig: 'viip',
  glUniform1ui__sig: 'vii',
  glUniform1uiv__sig: 'viip',
  glUniform2f__sig: 'viff',
  glUniform2fv__sig: 'viip',
  glUniform2i__sig: 'viii',
  glUniform2iv__sig: 'viip',
  glUniform2ui__sig: 'viii',
  glUniform2uiv__sig: 'viip',
  glUniform3f__sig: 'vifff',
  glUniform3fv__sig: 'viip',
  glUniform3i__sig: 'viiii',
  glUniform3iv__sig: 'viip',
  glUniform3ui__sig: 'viiii',
  glUniform3uiv__sig: 'viip',
  glUniform4f__sig: 'viffff',
  glUniform4fv__sig: 'viip',
  glUniform4i__sig: 'viiiii',
  glUniform4iv__sig: 'viip',
  glUniform4ui__sig: 'viiiii',
  glUniform4uiv__sig: 'viip',
  glUniformBlockBinding__sig: 'viii',
  glUniformMatrix2fv__sig: 'viiip',
  glUniformMatrix2x3fv__sig: 'viiip',
  glUniformMatrix2x4fv__sig: 'viiip',
  glUniformMatrix3fv__sig: 'viiip',
  glUniformMatrix3x2fv__sig: 'viiip',
  glUniformMatrix3x4fv__sig: 'viiip',
  glUniformMatrix4fv__sig: 'viiip',
  glUniformMatrix4x2fv__sig: 'viiip',
  glUniformMatrix4x3fv__sig: 'viiip',
  glUnmapBuffer__sig: 'ii',
  glUseProgram__sig: 'vi',
  glValidateProgram__sig: 'vi',
  glVertex2f__sig: 'vff',
  glVertex2fv__sig: 'vp',
  glVertex2i__sig: 'vii',
  glVertex3f__sig: 'vfff',
  glVertex3fv__sig: 'vp',
  glVertex3i__sig: 'viii',
  glVertex4f__sig: 'vffff',
  glVertex4fv__sig: 'vp',
  glVertex4i__sig: 'viiii',
  glVertexAttrib1f__sig: 'vif',
  glVertexAttrib1fv__sig: 'vip',
  glVertexAttrib2f__sig: 'viff',
  glVertexAttrib2fv__sig: 'vip',
  glVertexAttrib3f__sig: 'vifff',
  glVertexAttrib3fv__sig: 'vip',
  glVertexAttrib4f__sig: 'viffff',
  glVertexAttrib4fv__sig: 'vip',
  glVertexAttribDivisor__sig: 'vii',
  glVertexAttribI4i__sig: 'viiiii',
  glVertexAttribI4iv__sig: 'vip',
  glVertexAttribI4ui__sig: 'viiiii',
  glVertexAttribI4uiv__sig: 'vip',
  glVertexAttribIPointer__sig: 'viiiip',
  glVertexAttribPointer__sig: 'viiiiip',
  glVertexPointer__sig: 'viiip',
  glViewport__sig: 'viiii',
  glWaitSync__sig: 'vpij',
  glewGetErrorString__sig: 'pi',
  glewGetExtension__sig: 'ip',
  glewGetString__sig: 'pi',
  glewInit__sig: 'i',
  glewIsSupported__sig: 'ip',
  glfwBroadcastCond__sig: 'vp',
  glfwCloseWindow__sig: 'v',
  glfwCreateCond__sig: 'p',
  glfwCreateCursor__sig: 'ppii',
  glfwCreateMutex__sig: 'p',
  glfwCreateStandardCursor__sig: 'pi',
  glfwCreateThread__sig: 'ipp',
  glfwCreateWindow__sig: 'piippp',
  glfwDefaultWindowHints__sig: 'v',
  glfwDestroyCond__sig: 'vp',
  glfwDestroyCursor__sig: 'vp',
  glfwDestroyMutex__sig: 'vp',
  glfwDestroyThread__sig: 'vi',
  glfwDestroyWindow__sig: 'vp',
  glfwDisable__sig: 'vi',
  glfwEnable__sig: 'vi',
  glfwExtensionSupported__sig: 'ip',
  glfwFocusWindow__sig: 'vp',
  glfwFreeImage__sig: 'vp',
  glfwGetClipboardString__sig: 'pp',
  glfwGetCurrentContext__sig: 'p',
  glfwGetCursorPos__sig: 'vppp',
  glfwGetDesktopMode__sig: 'vp',
  glfwGetFramebufferSize__sig: 'vppp',
  glfwGetGLVersion__sig: 'vppp',
  glfwGetGammaRamp__sig: 'pp',
  glfwGetInputMode__sig: 'ipi',
  glfwGetJoystickAxes__sig: 'pip',
  glfwGetJoystickButtons__sig: 'pip',
  glfwGetJoystickGUID__sig: 'pi',
  glfwGetJoystickHats__sig: 'pip',
  glfwGetJoystickName__sig: 'pi',
  glfwGetJoystickUserPointer__sig: 'pi',
  glfwGetKey__sig: 'ipi',
  glfwGetKeyName__sig: 'pii',
  glfwGetKeyScancode__sig: 'ii',
  glfwGetKey_v2__sig: 'ii',
  glfwGetMonitorContentScale__sig: 'vppp',
  glfwGetMonitorName__sig: 'pp',
  glfwGetMonitorPhysicalSize__sig: 'vppp',
  glfwGetMonitorPos__sig: 'vppp',
  glfwGetMonitorWorkarea__sig: 'vppppp',
  glfwGetMonitors__sig: 'pp',
  glfwGetMouseButton__sig: 'ipi',
  glfwGetMouseButton_v2__sig: 'ii',
  glfwGetMousePos__sig: 'vpp',
  glfwGetMouseWheel__sig: 'i',
  glfwGetNumberOfProcessors__sig: 'i',
  glfwGetPrimaryMonitor__sig: 'p',
  glfwGetProcAddress__sig: 'pp',
  glfwGetRequiredInstanceExtensions__sig: 'pp',
  glfwGetThreadID__sig: 'i',
  glfwGetTime__sig: 'd',
  glfwGetTimerFrequency__sig: 'j',
  glfwGetTimerValue__sig: 'j',
  glfwGetVersion__sig: 'vppp',
  glfwGetVersionString__sig: 'p',
  glfwGetVideoMode__sig: 'pp',
  glfwGetVideoModes__sig: 'ppp',
  glfwGetWindowAttrib__sig: 'ipi',
  glfwGetWindowContentScale__sig: 'vppp',
  glfwGetWindowFrameSize__sig: 'vppppp',
  glfwGetWindowMonitor__sig: 'pp',
  glfwGetWindowOpacity__sig: 'fp',
  glfwGetWindowParam__sig: 'ii',
  glfwGetWindowPos__sig: 'vppp',
  glfwGetWindowSize__sig: 'vppp',
  glfwGetWindowSize_v2__sig: 'vpp',
  glfwGetWindowUserPointer__sig: 'pp',
  glfwHideWindow__sig: 'vp',
  glfwIconifyWindow__sig: 'vp',
  glfwIconifyWindow_v2__sig: 'v',
  glfwInit__sig: 'i',
  glfwJoystickIsGamepad__sig: 'ii',
  glfwJoystickPresent__sig: 'ii',
  glfwLoadMemoryTexture2D__sig: 'ippi',
  glfwLoadTexture2D__sig: 'ipi',
  glfwLoadTextureImage2D__sig: 'ipi',
  glfwLockMutex__sig: 'vp',
  glfwMakeContextCurrent__sig: 'vp',
  glfwMaximizeWindow__sig: 'vp',
  glfwOpenWindow__sig: 'iiiiiiiiii',
  glfwOpenWindowHint__sig: 'vii',
  glfwPollEvents__sig: 'v',
  glfwPostEmptyEvent__sig: 'v',
  glfwRawMouseMotionSupported__sig: 'i',
  glfwReadImage__sig: 'ippi',
  glfwReadMemoryImage__sig: 'ipppi',
  glfwRequestWindowAttention__sig: 'vp',
  glfwRestoreWindow__sig: 'vp',
  glfwRestoreWindow_v2__sig: 'v',
  glfwSetCharCallback__sig: 'ppp',
  glfwSetCharCallback_v2__sig: 'vp',
  glfwSetCharModsCallback__sig: 'ppp',
  glfwSetClipboardString__sig: 'vpp',
  glfwSetCursor__sig: 'vpp',
  glfwSetCursorEnterCallback__sig: 'ppp',
  glfwSetCursorPos__sig: 'vpdd',
  glfwSetCursorPosCallback__sig: 'ppp',
  glfwSetDropCallback__sig: 'ppp',
  glfwSetErrorCallback__sig: 'pp',
  glfwSetFramebufferSizeCallback__sig: 'ppp',
  glfwSetGamma__sig: 'vpf',
  glfwSetGammaRamp__sig: 'vpp',
  glfwSetInputMode__sig: 'vpii',
  glfwSetJoystickCallback__sig: 'pp',
  glfwSetJoystickUserPointer__sig: 'vip',
  glfwSetKeyCallback__sig: 'ppp',
  glfwSetKeyCallback_v2__sig: 'vp',
  glfwSetMonitorCallback__sig: 'pp',
  glfwSetMouseButtonCallback__sig: 'ppp',
  glfwSetMouseButtonCallback_v2__sig: 'vp',
  glfwSetMousePos__sig: 'vii',
  glfwSetMousePosCallback__sig: 'vp',
  glfwSetMouseWheel__sig: 'vi',
  glfwSetMouseWheelCallback__sig: 'vp',
  glfwSetScrollCallback__sig: 'ppp',
  glfwSetTime__sig: 'vd',
  glfwSetWindowAspectRatio__sig: 'vpii',
  glfwSetWindowAttrib__sig: 'vpii',
  glfwSetWindowCloseCallback__sig: 'ppp',
  glfwSetWindowCloseCallback_v2__sig: 'vp',
  glfwSetWindowContentScaleCallback__sig: 'ppp',
  glfwSetWindowFocusCallback__sig: 'ppp',
  glfwSetWindowIcon__sig: 'vpip',
  glfwSetWindowIconifyCallback__sig: 'ppp',
  glfwSetWindowMaximizeCallback__sig: 'ppp',
  glfwSetWindowMonitor__sig: 'vppiiiii',
  glfwSetWindowOpacity__sig: 'vpf',
  glfwSetWindowPos__sig: 'vpii',
  glfwSetWindowPosCallback__sig: 'ppp',
  glfwSetWindowPos_v2__sig: 'vii',
  glfwSetWindowRefreshCallback__sig: 'ppp',
  glfwSetWindowRefreshCallback_v2__sig: 'vp',
  glfwSetWindowShouldClose__sig: 'vpi',
  glfwSetWindowSize__sig: 'vpii',
  glfwSetWindowSizeCallback__sig: 'ppp',
  glfwSetWindowSizeCallback_v2__sig: 'vp',
  glfwSetWindowSizeLimits__sig: 'vpiiii',
  glfwSetWindowSize_v2__sig: 'vii',
  glfwSetWindowTitle__sig: 'vpp',
  glfwSetWindowTitle_v2__sig: 'vp',
  glfwSetWindowUserPointer__sig: 'vpp',
  glfwShowWindow__sig: 'vp',
  glfwSignalCond__sig: 'vp',
  glfwSleep__sig: 'vd',
  glfwSwapBuffers__sig: 'vp',
  glfwSwapBuffers_v2__sig: 'v',
  glfwSwapInterval__sig: 'vi',
  glfwTerminate__sig: 'v',
  glfwUnlockMutex__sig: 'vp',
  glfwVulkanSupported__sig: 'i',
  glfwWaitCond__sig: 'vppd',
  glfwWaitEvents__sig: 'v',
  glfwWaitEventsTimeout__sig: 'vd',
  glfwWaitThread__sig: 'iii',
  glfwWindowHint__sig: 'vii',
  glfwWindowHintString__sig: 'vip',
  glfwWindowShouldClose__sig: 'ip',
  gluLookAt__sig: 'vddddddddd',
  gluOrtho2D__sig: 'vdddd',
  gluPerspective__sig: 'vdddd',
  gluProject__sig: 'idddpppppp',
  gluUnProject__sig: 'idddpppppp',
  glutCreateWindow__sig: 'ip',
  glutDestroyWindow__sig: 'vi',
  glutDisplayFunc__sig: 'vp',
  glutFullScreen__sig: 'v',
  glutGet__sig: 'ii',
  glutGetModifiers__sig: 'i',
  glutIdleFunc__sig: 'vp',
  glutInit__sig: 'vpp',
  glutInitDisplayMode__sig: 'vi',
  glutInitWindowPosition__sig: 'vii',
  glutInitWindowSize__sig: 'vii',
  glutKeyboardFunc__sig: 'vp',
  glutKeyboardUpFunc__sig: 'vp',
  glutMainLoop__sig: 'v',
  glutMotionFunc__sig: 'vp',
  glutMouseFunc__sig: 'vp',
  glutPassiveMotionFunc__sig: 'vp',
  glutPositionWindow__sig: 'vii',
  glutPostRedisplay__sig: 'v',
  glutReshapeFunc__sig: 'vp',
  glutReshapeWindow__sig: 'vii',
  glutSetCursor__sig: 'vi',
  glutSpecialFunc__sig: 'vp',
  glutSpecialUpFunc__sig: 'vp',
  glutSwapBuffers__sig: 'v',
  glutTimerFunc__sig: 'vipi',
  lineColor__sig: 'ipiiiii',
  lineRGBA__sig: 'ipiiiiiiii',
  llvm_eh_typeid_for__sig: 'vp',
  pixelRGBA__sig: 'ipiiiiii',
  proc_exit__sig: 'vi',
  random_get__sig: 'ipp',
  rectangleColor__sig: 'ipiiiii',
  rectangleRGBA__sig: 'ipiiiiiiii',
  rotozoomSurface__sig: 'ppddi',
  sched_yield__sig: 'i',
  setprotoent__sig: 'vi',
  strftime__sig: 'ppppp',
  strftime_l__sig: 'pppppp',
  strptime__sig: 'pppp',
  strptime_l__sig: 'ppppp',
  system__sig: 'ip',
  uuid_clear__sig: 'vp',
  uuid_compare__sig: 'ipp',
  uuid_copy__sig: 'vpp',
  uuid_generate__sig: 'vp',
  uuid_is_null__sig: 'ip',
  uuid_parse__sig: 'ipp',
  uuid_type__sig: 'ip',
  uuid_unparse__sig: 'vpp',
  uuid_unparse_lower__sig: 'vpp',
  uuid_unparse_upper__sig: 'vpp',
  uuid_variant__sig: 'ip',
  wgpuAdapterEnumerateFeatures__sig: 'ppp',
  wgpuAdapterGetLimits__sig: 'ipp',
  wgpuAdapterGetProperties__sig: 'vpp',
  wgpuAdapterHasFeature__sig: 'ipi',
  wgpuAdapterReference__sig: 'vp',
  wgpuAdapterRelease__sig: 'vp',
  wgpuAdapterRequestDevice__sig: 'vpppp',
  wgpuBindGroupLayoutReference__sig: 'vp',
  wgpuBindGroupLayoutRelease__sig: 'vp',
  wgpuBindGroupLayoutSetLabel__sig: 'vpp',
  wgpuBindGroupReference__sig: 'vp',
  wgpuBindGroupRelease__sig: 'vp',
  wgpuBindGroupSetLabel__sig: 'vpp',
  wgpuBufferDestroy__sig: 'vp',
  wgpuBufferGetConstMappedRange__sig: 'pppp',
  wgpuBufferGetMapState__sig: 'ip',
  wgpuBufferGetMappedRange__sig: 'pppp',
  wgpuBufferGetSize__sig: 'jp',
  wgpuBufferGetUsage__sig: 'ip',
  wgpuBufferMapAsync__sig: 'vpipppp',
  wgpuBufferReference__sig: 'vp',
  wgpuBufferRelease__sig: 'vp',
  wgpuBufferSetLabel__sig: 'vpp',
  wgpuBufferUnmap__sig: 'vp',
  wgpuCommandBufferReference__sig: 'vp',
  wgpuCommandBufferRelease__sig: 'vp',
  wgpuCommandBufferSetLabel__sig: 'vpp',
  wgpuCommandEncoderBeginComputePass__sig: 'ppp',
  wgpuCommandEncoderBeginRenderPass__sig: 'ppp',
  wgpuCommandEncoderClearBuffer__sig: 'vppjj',
  wgpuCommandEncoderCopyBufferToBuffer__sig: 'vppjpjj',
  wgpuCommandEncoderCopyBufferToTexture__sig: 'vpppp',
  wgpuCommandEncoderCopyTextureToBuffer__sig: 'vpppp',
  wgpuCommandEncoderCopyTextureToTexture__sig: 'vpppp',
  wgpuCommandEncoderFinish__sig: 'ppp',
  wgpuCommandEncoderInsertDebugMarker__sig: 'vpp',
  wgpuCommandEncoderPopDebugGroup__sig: 'vp',
  wgpuCommandEncoderPushDebugGroup__sig: 'vpp',
  wgpuCommandEncoderReference__sig: 'vp',
  wgpuCommandEncoderRelease__sig: 'vp',
  wgpuCommandEncoderResolveQuerySet__sig: 'vppiipj',
  wgpuCommandEncoderSetLabel__sig: 'vpp',
  wgpuCommandEncoderWriteTimestamp__sig: 'vppi',
  wgpuComputePassEncoderDispatchWorkgroups__sig: 'vpiii',
  wgpuComputePassEncoderDispatchWorkgroupsIndirect__sig: 'vppj',
  wgpuComputePassEncoderEnd__sig: 'vp',
  wgpuComputePassEncoderInsertDebugMarker__sig: 'vpp',
  wgpuComputePassEncoderPopDebugGroup__sig: 'vp',
  wgpuComputePassEncoderPushDebugGroup__sig: 'vpp',
  wgpuComputePassEncoderReference__sig: 'vp',
  wgpuComputePassEncoderRelease__sig: 'vp',
  wgpuComputePassEncoderSetBindGroup__sig: 'vpippp',
  wgpuComputePassEncoderSetLabel__sig: 'vpp',
  wgpuComputePassEncoderSetPipeline__sig: 'vpp',
  wgpuComputePassEncoderWriteTimestamp__sig: 'vppi',
  wgpuComputePipelineGetBindGroupLayout__sig: 'ppi',
  wgpuComputePipelineReference__sig: 'vp',
  wgpuComputePipelineRelease__sig: 'vp',
  wgpuComputePipelineSetLabel__sig: 'vpp',
  wgpuCreateInstance__sig: 'pp',
  wgpuDeviceCreateBindGroup__sig: 'ppp',
  wgpuDeviceCreateBindGroupLayout__sig: 'ppp',
  wgpuDeviceCreateBuffer__sig: 'ppp',
  wgpuDeviceCreateCommandEncoder__sig: 'ppp',
  wgpuDeviceCreateComputePipeline__sig: 'ppp',
  wgpuDeviceCreateComputePipelineAsync__sig: 'vpppp',
  wgpuDeviceCreatePipelineLayout__sig: 'ppp',
  wgpuDeviceCreateQuerySet__sig: 'ppp',
  wgpuDeviceCreateRenderBundleEncoder__sig: 'ppp',
  wgpuDeviceCreateRenderPipeline__sig: 'ppp',
  wgpuDeviceCreateRenderPipelineAsync__sig: 'vpppp',
  wgpuDeviceCreateSampler__sig: 'ppp',
  wgpuDeviceCreateShaderModule__sig: 'ppp',
  wgpuDeviceCreateSwapChain__sig: 'pppp',
  wgpuDeviceCreateTexture__sig: 'ppp',
  wgpuDeviceDestroy__sig: 'vp',
  wgpuDeviceEnumerateFeatures__sig: 'ppp',
  wgpuDeviceGetLimits__sig: 'ipp',
  wgpuDeviceGetQueue__sig: 'pp',
  wgpuDeviceHasFeature__sig: 'ipi',
  wgpuDevicePopErrorScope__sig: 'vppp',
  wgpuDevicePushErrorScope__sig: 'vpi',
  wgpuDeviceReference__sig: 'vp',
  wgpuDeviceRelease__sig: 'vp',
  wgpuDeviceSetLabel__sig: 'vpp',
  wgpuDeviceSetUncapturedErrorCallback__sig: 'vppp',
  wgpuGetProcAddress__sig: 'ppp',
  wgpuInstanceCreateSurface__sig: 'ppp',
  wgpuInstanceProcessEvents__sig: 'vp',
  wgpuInstanceReference__sig: 'vp',
  wgpuInstanceRelease__sig: 'vp',
  wgpuInstanceRequestAdapter__sig: 'vpppp',
  wgpuPipelineLayoutReference__sig: 'vp',
  wgpuPipelineLayoutRelease__sig: 'vp',
  wgpuPipelineLayoutSetLabel__sig: 'vpp',
  wgpuQuerySetDestroy__sig: 'vp',
  wgpuQuerySetGetCount__sig: 'ip',
  wgpuQuerySetGetType__sig: 'ip',
  wgpuQuerySetReference__sig: 'vp',
  wgpuQuerySetRelease__sig: 'vp',
  wgpuQuerySetSetLabel__sig: 'vpp',
  wgpuQueueOnSubmittedWorkDone__sig: 'vpjpp',
  wgpuQueueReference__sig: 'vp',
  wgpuQueueRelease__sig: 'vp',
  wgpuQueueSetLabel__sig: 'vpp',
  wgpuQueueSubmit__sig: 'vppp',
  wgpuQueueWriteBuffer__sig: 'vppjpp',
  wgpuQueueWriteTexture__sig: 'vpppppp',
  wgpuRenderBundleEncoderDraw__sig: 'vpiiii',
  wgpuRenderBundleEncoderDrawIndexed__sig: 'vpiiiii',
  wgpuRenderBundleEncoderDrawIndexedIndirect__sig: 'vppj',
  wgpuRenderBundleEncoderDrawIndirect__sig: 'vppj',
  wgpuRenderBundleEncoderFinish__sig: 'ppp',
  wgpuRenderBundleEncoderInsertDebugMarker__sig: 'vpp',
  wgpuRenderBundleEncoderPopDebugGroup__sig: 'vp',
  wgpuRenderBundleEncoderPushDebugGroup__sig: 'vpp',
  wgpuRenderBundleEncoderReference__sig: 'vp',
  wgpuRenderBundleEncoderRelease__sig: 'vp',
  wgpuRenderBundleEncoderSetBindGroup__sig: 'vpippp',
  wgpuRenderBundleEncoderSetIndexBuffer__sig: 'vppijj',
  wgpuRenderBundleEncoderSetLabel__sig: 'vpp',
  wgpuRenderBundleEncoderSetPipeline__sig: 'vpp',
  wgpuRenderBundleEncoderSetVertexBuffer__sig: 'vpipjj',
  wgpuRenderBundleReference__sig: 'vp',
  wgpuRenderBundleRelease__sig: 'vp',
  wgpuRenderBundleSetLabel__sig: 'vpp',
  wgpuRenderPassEncoderBeginOcclusionQuery__sig: 'vpi',
  wgpuRenderPassEncoderDraw__sig: 'vpiiii',
  wgpuRenderPassEncoderDrawIndexed__sig: 'vpiiiii',
  wgpuRenderPassEncoderDrawIndexedIndirect__sig: 'vppj',
  wgpuRenderPassEncoderDrawIndirect__sig: 'vppj',
  wgpuRenderPassEncoderEnd__sig: 'vp',
  wgpuRenderPassEncoderEndOcclusionQuery__sig: 'vp',
  wgpuRenderPassEncoderExecuteBundles__sig: 'vppp',
  wgpuRenderPassEncoderInsertDebugMarker__sig: 'vpp',
  wgpuRenderPassEncoderPopDebugGroup__sig: 'vp',
  wgpuRenderPassEncoderPushDebugGroup__sig: 'vpp',
  wgpuRenderPassEncoderReference__sig: 'vp',
  wgpuRenderPassEncoderRelease__sig: 'vp',
  wgpuRenderPassEncoderSetBindGroup__sig: 'vpippp',
  wgpuRenderPassEncoderSetBlendConstant__sig: 'vpp',
  wgpuRenderPassEncoderSetIndexBuffer__sig: 'vppijj',
  wgpuRenderPassEncoderSetLabel__sig: 'vpp',
  wgpuRenderPassEncoderSetPipeline__sig: 'vpp',
  wgpuRenderPassEncoderSetScissorRect__sig: 'vpiiii',
  wgpuRenderPassEncoderSetStencilReference__sig: 'vpi',
  wgpuRenderPassEncoderSetVertexBuffer__sig: 'vpipjj',
  wgpuRenderPassEncoderSetViewport__sig: 'vpffffff',
  wgpuRenderPassEncoderWriteTimestamp__sig: 'vppi',
  wgpuRenderPipelineGetBindGroupLayout__sig: 'ppi',
  wgpuRenderPipelineReference__sig: 'vp',
  wgpuRenderPipelineRelease__sig: 'vp',
  wgpuRenderPipelineSetLabel__sig: 'vpp',
  wgpuSamplerReference__sig: 'vp',
  wgpuSamplerRelease__sig: 'vp',
  wgpuSamplerSetLabel__sig: 'vpp',
  wgpuShaderModuleGetCompilationInfo__sig: 'vppp',
  wgpuShaderModuleReference__sig: 'vp',
  wgpuShaderModuleRelease__sig: 'vp',
  wgpuShaderModuleSetLabel__sig: 'vpp',
  wgpuSurfaceGetPreferredFormat__sig: 'ipp',
  wgpuSurfaceReference__sig: 'vp',
  wgpuSurfaceRelease__sig: 'vp',
  wgpuSwapChainGetCurrentTextureView__sig: 'pp',
  wgpuSwapChainPresent__sig: 'vp',
  wgpuSwapChainReference__sig: 'vp',
  wgpuSwapChainRelease__sig: 'vp',
  wgpuTextureCreateView__sig: 'ppp',
  wgpuTextureDestroy__sig: 'vp',
  wgpuTextureGetDepthOrArrayLayers__sig: 'ip',
  wgpuTextureGetDimension__sig: 'ip',
  wgpuTextureGetFormat__sig: 'ip',
  wgpuTextureGetHeight__sig: 'ip',
  wgpuTextureGetMipLevelCount__sig: 'ip',
  wgpuTextureGetSampleCount__sig: 'ip',
  wgpuTextureGetUsage__sig: 'ip',
  wgpuTextureGetWidth__sig: 'ip',
  wgpuTextureReference__sig: 'vp',
  wgpuTextureRelease__sig: 'vp',
  wgpuTextureSetLabel__sig: 'vpp',
  wgpuTextureViewReference__sig: 'vp',
  wgpuTextureViewRelease__sig: 'vp',
  wgpuTextureViewSetLabel__sig: 'vpp',
  zoomSurface__sig: 'ppddi',
}

// We have to merge with `allowMissing` since this file contains signatures
// for functions that might not exist in all build configurations.
addToLibrary(sigs, {allowMissing: true});
