// Copyright 2016 The Emscripten Authors.  All rights reserved.
// Emscripten is available under two separate licenses, the MIT license and the
// University of Illinois/NCSA Open Source License.  Both these licenses can be
// found in the LICENSE file.

#include <emscripten.h>

extern "C" {

EMSCRIPTEN_KEEPALIVE int add(int x, int y) {
  return x + y;
}

}
