module.exports={A:{A:{"2":"J E F G A B PC"},B:{"1":"0 1 2 3 4 5 6 N O P Q R S T U V W X Y Z a b c d e f g h i j k l m n t u v w x y z D","2":"C K L H M"},C:{"1":"0 1 2 3 4 5 6 7 8 9 I J E F G A B C K L H M N O o p q r AB BB CB DB EB FB GB HB IB JB KB LB MB NB OB PB QB RB SB TB UB VB WB XB YB ZB aB bB cB dB eB fB gB hB 5B iB 6B jB kB lB mB nB oB pB qB rB sB tB s uB vB wB xB yB P Q R 7B S T U V W X Y Z a b c d e f g h i j k l m n t u v w x y z D 8B 9B AC RC SC TC","2":"QC 4B"},D:{"1":"0 1 2 3 4 5 6 7 8 9 I J E F G A B C K L H M N O o p q r AB BB CB DB EB FB GB HB IB JB KB LB MB NB OB PB QB RB SB TB UB VB WB XB YB ZB aB bB cB dB eB fB gB hB 5B iB 6B jB kB lB mB nB oB pB qB rB sB tB s uB vB wB xB yB P Q R S T U V W X Y Z a b c d e f g h i j k l m n t u v w x y z D 8B 9B AC"},E:{"2":"7 I J E F G A B C K L UC BC VC WC XC YC CC zB 0B DC","132":"H ZC aC EC FC 1B bC 2B GC HC IC JC KC cC 3B LC MC dC"},F:{"1":"8 9 B C H M N O o p q r AB BB CB DB EB FB GB HB IB JB KB LB MB NB OB PB QB RB SB TB UB VB WB XB YB ZB aB bB cB dB eB fB gB hB iB jB kB lB mB nB oB pB qB rB sB tB s uB vB wB xB yB P Q R 7B S T U V W X Y Z a b c d e f g h i j k l m n gC hC zB NC iC 0B","2":"G eC fC"},G:{"2":"F BC jC OC kC lC mC nC oC pC qC rC sC tC uC vC wC xC yC zC 0C 1C 2C EC FC 1B 3C 2B GC HC IC JC KC 4C 3B LC MC"},H:{"2":"5C"},I:{"1":"4B I D 8C 9C OC AD BD","16":"6C 7C"},J:{"1":"A","2":"E"},K:{"1":"B C s zB NC 0B","2":"A"},L:{"1":"D"},M:{"1":"D"},N:{"2":"A B"},O:{"1":"1B"},P:{"1":"I o p q r CD DD ED FD GD CC HD ID JD KD LD 2B 3B MD ND"},Q:{"1":"DC"},R:{"1":"OD"},S:{"1":"PD QD"}},B:6,C:"Ogg Vorbis audio format",D:true};
