
var a;a||(a=typeof Module !== 'undefined' ? Module : {});var k={},m;for(m in a)a.hasOwnProperty(m)&&(k[m]=a[m]);function p(b,c){throw c;}var q=!1,r=!1,t=!1,u=!1;q="object"===typeof window;r="function"===typeof importScripts;t="object"===typeof process&&"object"===typeof process.versions&&"string"===typeof process.versions.node;u=!q&&!t&&!r;var w="",y,z,A,B;
if(t)w=r?require("path").dirname(w)+"/":__dirname+"/",y=function(b,c){A||(A=require("fs"));B||(B=require("path"));b=B.normalize(b);return A.readFileSync(b,c?null:"utf8")},z=function(b){b=y(b,!0);b.buffer||(b=new Uint8Array(b));b.buffer||C("Assertion failed: undefined");return b},1<process.argv.length&&process.argv[1].replace(/\\/g,"/"),process.argv.slice(2),"undefined"!==typeof module&&(module.exports=a),process.on("uncaughtException",function(b){if(!(b instanceof D))throw b;}),process.on("unhandledRejection",
C),p=function(b){process.exit(b)},a.inspect=function(){return"[Emscripten Module object]"};else if(u)"undefined"!=typeof read&&(y=function(b){return read(b)}),z=function(b){if("function"===typeof readbuffer)return new Uint8Array(readbuffer(b));b=read(b,"binary");"object"===typeof b||C("Assertion failed: undefined");return b},"function"===typeof quit&&(p=function(b){quit(b)}),"undefined"!==typeof print&&("undefined"===typeof console&&(console={}),console.log=print,console.warn=console.error="undefined"!==
typeof printErr?printErr:print);else if(q||r)r?w=self.location.href:document.currentScript&&(w=document.currentScript.src),w=0!==w.indexOf("blob:")?w.substr(0,w.lastIndexOf("/")+1):"",y=function(b){var c=new XMLHttpRequest;c.open("GET",b,!1);c.send(null);return c.responseText},r&&(z=function(b){var c=new XMLHttpRequest;c.open("GET",b,!1);c.responseType="arraybuffer";c.send(null);return new Uint8Array(c.response)});var aa=a.print||console.log.bind(console),F=a.printErr||console.warn.bind(console);
for(m in k)k.hasOwnProperty(m)&&(a[m]=k[m]);k=null;a.quit&&(p=a.quit);var G;a.wasmBinary&&(G=a.wasmBinary);var noExitRuntime;a.noExitRuntime&&(noExitRuntime=a.noExitRuntime);"object"!==typeof WebAssembly&&C("no native wasm support detected");var H,I=new WebAssembly.Table({initial:4,maximum:4,element:"anyfunc"}),J=!1,ba="undefined"!==typeof TextDecoder?new TextDecoder("utf8"):void 0,K,L,M,N=a.INITIAL_MEMORY||16777216;a.wasmMemory?H=a.wasmMemory:H=new WebAssembly.Memory({initial:N/65536,maximum:N/65536});
H&&(K=H.buffer);N=K.byteLength;var O=K;K=O;a.HEAP8=new Int8Array(O);a.HEAP16=new Int16Array(O);a.HEAP32=M=new Int32Array(O);a.HEAPU8=L=new Uint8Array(O);a.HEAPU16=new Uint16Array(O);a.HEAPU32=new Uint32Array(O);a.HEAPF32=new Float32Array(O);a.HEAPF64=new Float64Array(O);var ca=[],da=[],ea=[],fa=[];function ha(){var b=a.preRun.shift();ca.unshift(b)}var P=0,Q=null,R=null;a.preloadedImages={};a.preloadedAudios={};
function C(b){if(a.onAbort)a.onAbort(b);F(b);J=!0;throw new WebAssembly.RuntimeError("abort("+b+"). Build with -s ASSERTIONS=1 for more info.");}function S(b){var c=T;return String.prototype.startsWith?c.startsWith(b):0===c.indexOf(b)}function ia(){return S("data:application/octet-stream;base64,")}var T="test_emsize.wasm";if(!ia()){var la=T;T=a.locateFile?a.locateFile(la,w):w+la}
function ma(){try{if(G)return new Uint8Array(G);if(z)return z(T);throw"both async and sync fetching of the wasm failed";}catch(b){C(b)}}function na(){return G||!q&&!r||"function"!==typeof fetch||S("file://")?Promise.resolve().then(ma):fetch(T,{credentials:"same-origin"}).then(function(b){if(!b.ok)throw"failed to load wasm binary file at '"+T+"'";return b.arrayBuffer()}).catch(function(){return ma()})}
function U(b){for(;0<b.length;){var c=b.shift();if("function"==typeof c)c(a);else{var d=c.h;"number"===typeof d?void 0===c.g?I.get(d)():I.get(d)(c.g):d(void 0===c.g?null:c.g)}}}var oa=[null,[],[]];da.push({h:function(){pa()}});
var qa={b:I,d:function(b,c,d){L.copyWithin(b,c,c+d)},c:function(b,c,d,e){for(var f=0,h=0;h<d;h++){for(var ra=M[c+8*h>>2],ja=M[c+(8*h+4)>>2],V=0;V<ja;V++){var E=L[ra+V],W=oa[b];if(0===E||10===E){E=1===b?aa:F;var l=W;for(var n=0,v=n+NaN,x=n;l[x]&&!(x>=v);)++x;if(16<x-n&&l.subarray&&ba)l=ba.decode(l.subarray(n,x));else{for(v="";n<x;){var g=l[n++];if(g&128){var X=l[n++]&63;if(192==(g&224))v+=String.fromCharCode((g&31)<<6|X);else{var ka=l[n++]&63;g=224==(g&240)?(g&15)<<12|X<<6|ka:(g&7)<<18|X<<12|ka<<6|
l[n++]&63;65536>g?v+=String.fromCharCode(g):(g-=65536,v+=String.fromCharCode(55296|g>>10,56320|g&1023))}}else v+=String.fromCharCode(g)}l=v}E(l);W.length=0}else W.push(E)}f+=ja}M[e>>2]=f;return 0},a:H};
(function(){function b(f){console.log("receiveInstance");a.asm=f.exports;P--;a.monitorRunDependencies&&a.monitorRunDependencies(P);0==P&&(null!==Q&&(clearInterval(Q),Q=null),R&&(f=R,R=null,f()))}function c(f){b(f.instance)}function d(f){return na().then(function(h){return WebAssembly.instantiate(h,e)}).then(f,function(h){F("failed to asynchronously prepare wasm: "+h);C(h)})}var e={a:qa};P++;a.monitorRunDependencies&&a.monitorRunDependencies(P);if(a.instantiateWasm)try{return a.instantiateWasm(e,b)}catch(f){return F("Module.instantiateWasm callback failed with error: "+
f),!1}(function(){if(G||"function"!==typeof WebAssembly.instantiateStreaming||ia()||S("file://")||"function"!==typeof fetch)return d(c);fetch(T,{credentials:"same-origin"}).then(function(f){return WebAssembly.instantiateStreaming(f,e).then(c,function(h){F("wasm streaming compile failed: "+h);F("falling back to ArrayBuffer instantiation");return d(c)})})})();return{}})();var pa=a.___wasm_call_ctors=function(){return(pa=a.___wasm_call_ctors=a.asm.e).apply(null,arguments)};
a._main=function(){return(a._main=a.asm.f).apply(null,arguments)};a.___heap_base=5245632;var Y;function D(b){this.name="ExitStatus";this.message="Program terminated with exit("+b+")";this.status=b}R=function sa(){Y||Z();Y||(R=sa)};
function Z(){function b(){if(!Y&&(Y=!0,a.calledRun=!0,!J)){U(da);U(ea);if(a.onRuntimeInitialized)a.onRuntimeInitialized();if(ta){var c=a._main;try{var d=c(0,0);if(!noExitRuntime||0!==d){if(!noExitRuntime){if(a.onExit)a.onExit(d);J=!0}p(d,new D(d))}}catch(e){e instanceof D||("unwind"==e?noExitRuntime=!0:((c=e)&&"object"===typeof e&&e.stack&&(c=[e,e.stack]),F("exception thrown: "+c),p(1,e)))}finally{}}if(a.postRun)for("function"==typeof a.postRun&&(a.postRun=[a.postRun]);a.postRun.length;)c=a.postRun.shift(),
fa.unshift(c);U(fa)}}if(!(0<P)){if(a.preRun)for("function"==typeof a.preRun&&(a.preRun=[a.preRun]);a.preRun.length;)ha();U(ca);0<P||(a.setStatus?(a.setStatus("Running..."),setTimeout(function(){setTimeout(function(){a.setStatus("")},1);b()},1)):b())}}a.run=Z;if(a.preInit)for("function"==typeof a.preInit&&(a.preInit=[a.preInit]);0<a.preInit.length;)a.preInit.pop()();var ta=!0;a.noInitialRun&&(ta=!1);noExitRuntime=!0;Z();
