module.exports={A:{A:{"2":"J E F G A B PC"},B:{"1":"0 1 2 3 4 5 6 Y Z a b c d e f g h i j k l m n t u v w x y z D","2":"C K L H M N O","1028":"W X","1540":"P Q R S T U V"},C:{"1":"0 1 2 3 4 5 6 nB oB pB qB rB sB tB s uB vB wB xB yB P Q R 7B S T U V W X Y Z a b c d e f g h i j k l m n t u v w x y z D 8B 9B AC RC","2":"QC","164":"7 8 9 4B I J E F G A B C K L H M N O o p q r AB BB CB DB EB FB GB HB IB JB KB LB MB NB OB PB SC TC","1540":"QB RB SB TB UB VB WB XB YB ZB aB bB cB dB eB fB gB hB 5B iB 6B jB kB lB mB"},D:{"1":"0 1 2 3 4 5 6 Y Z a b c d e f g h i j k l m n t u v w x y z D 8B 9B AC","292":"7 8 9 I J E F G A B C K L H M N O o p q r AB BB CB DB EB FB GB HB IB JB KB LB MB NB OB PB QB RB SB TB UB VB WB XB YB ZB aB bB cB dB eB fB gB hB 5B iB 6B jB kB lB mB nB oB pB","1028":"W X","1540":"qB rB sB tB s uB vB wB xB yB P Q R S T U V"},E:{"1":"H aC EC FC 1B bC 2B GC HC IC JC KC cC 3B LC MC dC","292":"7 I J E F G A B C UC BC VC WC XC YC CC zB","1540":"K L 0B DC","3076":"ZC"},F:{"1":"wB xB yB P Q R 7B S T U V W X Y Z a b c d e f g h i j k l m n","2":"G B C eC fC gC hC zB NC iC 0B","292":"8 9 H M N O o p q r AB BB CB DB EB FB GB HB IB JB KB LB MB NB OB PB QB RB SB TB UB VB WB XB YB ZB aB bB cB dB eB","1028":"uB vB","1540":"fB gB hB iB jB kB lB mB nB oB pB qB rB sB tB s"},G:{"1":"2C EC FC 1B 3C 2B GC HC IC JC KC 4C 3B LC MC","292":"F BC jC OC kC lC mC nC oC pC qC rC sC tC uC","1540":"vC wC xC yC zC 0C","3076":"1C"},H:{"2":"5C"},I:{"1":"D","292":"4B I 6C 7C 8C 9C OC AD BD"},J:{"292":"E A"},K:{"1":"s","2":"A B C zB NC 0B"},L:{"1":"D"},M:{"1":"D"},N:{"2":"A B"},O:{"1":"1B"},P:{"1":"o p q r LD 2B 3B MD ND","292":"I CD DD ED FD GD","1540":"CC HD ID JD KD"},Q:{"1540":"DC"},R:{"1":"OD"},S:{"1":"QD","1540":"PD"}},B:5,C:"CSS Logical Properties",D:true};
