module.exports={A:{A:{"2":"J E F G PC","130":"A B"},B:{"1":"0 1 2 3 4 5 6 z D","130":"C K L H M N O P Q R S T U V W X Y Z a b c d e f g h i j k l m n t u v w x y"},C:{"1":"0 1 2 3 4 5 6 JB KB LB MB NB OB PB QB RB SB TB UB VB WB XB YB ZB aB bB cB dB eB fB gB hB 5B iB 6B jB kB lB mB nB oB pB qB rB sB tB s uB vB wB xB yB P Q R 7B S T U V W X Y Z a b c d e f g h i j k l m n t u v w x y z D 8B 9B AC RC","2":"QC 4B SC TC","130":"7 8 I J E F G A B C K L H M N O o p q r","322":"9 AB BB CB DB EB FB GB HB IB"},D:{"1":"0 1 2 3 4 5 6 z D 8B 9B AC","2":"7 I J E F G A B C K L H","130":"8 9 M N O o p q r AB BB CB DB EB FB GB HB IB JB KB LB MB NB OB PB QB RB SB TB UB VB WB XB YB ZB aB bB cB dB eB fB gB hB 5B iB 6B jB kB lB mB nB oB pB qB rB sB tB s uB vB wB xB yB P Q R S T U V W X Y Z a b c d e f g h i j k l m n t u v w x y"},E:{"1":"A B C K L H YC CC zB 0B DC ZC aC EC FC 1B bC 2B GC HC IC JC KC cC 3B LC MC dC","2":"E F G UC BC WC XC","130":"7 I J VC"},F:{"1":"h i j k l m n","2":"G B C eC fC gC hC zB NC iC 0B","130":"8 9 H M N O o p q r AB BB CB DB EB FB GB HB IB JB KB LB MB NB OB PB QB RB SB TB UB VB WB XB YB ZB aB bB cB dB eB fB gB hB iB jB kB lB mB nB oB pB qB rB sB tB s uB vB wB xB yB P Q R 7B S T U V W X Y Z a b c d e f g"},G:{"1":"pC qC rC sC tC uC vC wC xC yC zC 0C 1C 2C EC FC 1B 3C 2B GC HC IC JC KC 4C 3B LC MC","2":"F BC mC nC oC","130":"jC OC kC lC"},H:{"2":"5C"},I:{"1":"D","2":"4B I 6C 7C 8C 9C OC","130":"AD BD"},J:{"2":"E","130":"A"},K:{"2":"A B C zB NC 0B","130":"s"},L:{"1":"D"},M:{"1":"D"},N:{"2":"A B"},O:{"130":"1B"},P:{"1":"q r","130":"I o p CD DD ED FD GD CC HD ID JD KD LD 2B 3B MD ND"},Q:{"130":"DC"},R:{"130":"OD"},S:{"1":"PD QD"}},B:5,C:"CSS font-variant-alternates",D:true};
